﻿using System;
using System.Runtime.InteropServices;
using System.Threading;
using Microsoft.Office.Core;
using GSpreadCOM.Domain;

namespace GSpreadCOM
{
	[TypeLibType(TypeLibTypeFlags.FCanCreate | TypeLibTypeFlags.FDual | TypeLibTypeFlags.FDispatchable)]
	//[ClassInterface(ClassInterfaceType.AutoDual)]
	[ComSourceInterfaces("Microsoft.Office.Interop.Excel.AppEvents")]
	[Guid("F0024500-F000-F000-C000-F00000000046")] //should be generated, I've just patched original a bit
	[ComVisible(true)]
	public class Application : Microsoft.Office.Interop.Excel._Application, Microsoft.Office.Interop.Excel.Application, Microsoft.Office.Interop.Excel.AppEvents_Event
	{
		internal ApplicationService Service { get; private set; }

		public Application()
		{
			Config.Initialize();

			DecimalSeparator = Thread.CurrentThread.CurrentCulture.NumberFormat.NumberDecimalSeparator;
			ThousandsSeparator = Thread.CurrentThread.CurrentCulture.NumberFormat.NumberGroupSeparator;

			Service = new ApplicationService();
			Workbooks = new Workbooks(this);
		}

		#region properties

		/// <summary>
		/// Returns a Range object that represents the active cell in the active worksheet. Read-only.
		/// </summary>
		[DispId(305)]
		public Microsoft.Office.Interop.Excel.Range ActiveCell
		{
			get
			{
				return ActiveSheet != null ? (ActiveSheet as Worksheet).ActiveCell : null;
			}
		}

		/// <summary>
		/// Returns an object that represents the active sheet (the sheet on top) in the active workbook. Returns null if no sheet is active.
		/// </summary>
		[DispId(307)]
		public dynamic ActiveSheet
		{
			get
			{
				return ActiveWorkbook != null ? ActiveWorkbook.ActiveSheet : null;
			}
			set
			{
				Worksheet active = value;
				var workbooks = Workbooks as Workbooks;

				if (active != null)
				{
					workbooks.Active = active.Worksheets.Workbook;
					(workbooks.Active.Worksheets as Worksheets).Active = active;
				}
				else
				{
					workbooks.Active = null;
				}
			}
		}

		/// <summary>
		/// Returns a Workbook object that represents the active Google sheet. Read-only.
		/// </summary>
		[DispId(308)]
		public Microsoft.Office.Interop.Excel.Workbook ActiveWorkbook
		{
			get
			{
				return (Workbooks as Workbooks).Active;
			}
			set
			{
				(Workbooks as Workbooks).Active = value as Workbook;
			}
		}

		/// <summary>
		/// Returns a Range object that represents all the cells on the active worksheet.
		/// </summary>
		[DispId(238)]
		public Microsoft.Office.Interop.Excel.Range Cells
		{
			get { return ActiveSheet != null ? ActiveSheet.Cells : null; }
		}

		/// <summary>
		/// Returns a Range object that represents all the columns on the active worksheet.
		/// </summary>
		[DispId(241)]
		public Microsoft.Office.Interop.Excel.Range Columns
		{
			get { return ActiveSheet != null ? ActiveSheet.Columns : null; }
		}

		/// <summary>
		/// Sets or returns the character used for the decimal separator as a String. Read/write.
		/// </summary>
		[DispId(1809)]
		public string DecimalSeparator { get; set; }

		/// <summary>
		/// Returns a Range object that represents a cell or a range of cells.
		/// </summary>
		/// <param name="Cell1"></param>
		/// <param name="Cell2"></param>
		/// <returns></returns>
		[DispId(197)]
		public Microsoft.Office.Interop.Excel.Range get_Range(object Cell1, [Optional]object Cell2)
		{
			return ActiveSheet != null ? ActiveSheet.get_Range(Cell1, Cell2) : null;
		}

		/// <summary>
		/// Returns a Range object that represents all the rows on the active worksheet. Read-only.
		/// </summary>
		[DispId(258)]
		public Microsoft.Office.Interop.Excel.Range Rows
		{
			get { return ActiveSheet != null ? ActiveSheet.Rows : null; }
		}

		/// <summary>
		/// Returns a Sheets collection that represents all the sheets in the active workbook. Read-only.
		/// </summary>
		[DispId(485)]
		public Microsoft.Office.Interop.Excel.Sheets Sheets
		{
			get { return ActiveWorkbook != null ? ActiveWorkbook.Sheets : null; }
		}

		/// <summary>
		/// Sets or returns the character used for the thousands separator as a String. Read/write.
		/// </summary>
		[DispId(1810)]
		public string ThousandsSeparator { get; set; }

		/// <summary>
		/// Returns a Workbooks collection that represents all the open workbooks. Read-only.
		/// </summary>
		[DispId(572)]
		public Microsoft.Office.Interop.Excel.Workbooks Workbooks { get; private set; }

		/// <summary>
		/// Returns a Sheets collection that represents all the worksheets in the active workbook. Read-only.
		/// </summary>
		[DispId(494)]
		public Microsoft.Office.Interop.Excel.Sheets Worksheets
		{
			get { return ActiveWorkbook != null ? ActiveWorkbook.Worksheets : null; }
		}

		#endregion

		#region methods

		/// <summary>
		/// OAuth 2 web authorization for Google Apps Script, Drive API and Sheets API
		/// </summary>
		/// <param name="Name">Random user name</param>
		/// <param name="Password">Client ID and Secret in format: client_id|client_secret</param>
		/// <param name="DownloadNewMail">Google Apps script id</param>
		[LCIDConversion(0)]
		[DispId(943)]
		public void MailLogon([Optional]object Name, [Optional]object Password, [Optional]object DownloadNewMail)
		{
			string clientId = "";
			string clientSecret = "";

			string[] parts = (Password ?? "").ToString().Split('|');
			if (parts.Length == 2)
			{
				clientId = parts[0];
				clientSecret = parts[1];
			}

			Service.GoogleServicesAuthorization((Name ?? "").ToString(), clientId, clientSecret, (DownloadNewMail ?? "").ToString());
		}

		/// <summary>
		/// Revoke Google authorization token
		/// </summary>
		[DispId(302)]
		public void Quit() {
			Service.RevokeAuthorizationToken();
		}

		/// <summary>
		/// If Filename specified, copies active workbook (google spreadsheet), otherwise does nothing
		/// </summary>
		/// <param name="Filename"></param>
		[LCIDConversion(1)]
		[TypeLibFunc(64)]
		[DispId(283)]
		public void Save([Optional]object Filename)
		{
		    SaveWorkspace(Filename);
		}

		/// <summary>
		/// If Filename specified, copies active workbook (google spreadsheet), otherwise does nothing
		/// </summary>
		/// <param name="Filename"></param>
		[LCIDConversion(1)]
		[DispId(212)]
		public void SaveWorkspace([Optional]object Filename)
		{
		    if (ActiveWorkbook == null)
		    {
		        throw new InvalidOperationException("There is no active workbook to save");
		    }

			if (Filename is String)
			{
				ActiveWorkbook.SaveAs(Filename);
			}
			else
			{
				ActiveWorkbook.Save();
			}
		}

		#endregion

		#region questionable

		[LCIDConversion(0)]
		[DispId(279)]
		public void Calculate()
		{
			throw new NotImplementedException();
		}

		[DispId(1805)]
		public void CalculateFull()
		{
			throw new NotImplementedException();
		}

		[DispId(1945)]
		public void CalculateFullRebuild()
		{
			throw new NotImplementedException();
		}

		[DispId(2387)]
		public void CalculateUntilAsyncQueriesDone()
		{
			throw new NotImplementedException();
		}

		[DispId(316)]
		public Microsoft.Office.Interop.Excel.XlCalculation Calculation
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1086)]
		[LCIDConversion(1)]
		public double CentimetersToPoints(double Centimeters)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(1)]
		[DispId(1)]
		public dynamic Evaluate(object Name)
		{
			throw new NotImplementedException();
		}

		[DispId(1087)]
		[LCIDConversion(1)]
		public double InchesToPoints(double Inches)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(0)]
		[DispId(945)]
		public void MailLogoff()
		{
			throw new NotImplementedException();
		}

		[DispId(2375)]
		public int MeasurementUnit
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1565)]
		public NewFile NewWorkbook
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(147)]
		public dynamic Selection
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(924)]
		public string StandardFont
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(925)]
		public double StandardFontSize
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1962)]
		public Microsoft.Office.Interop.Excel.Range ThisCell
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(778)]
		public Microsoft.Office.Interop.Excel.Workbook ThisWorkbook
		{
			get { throw new NotImplementedException(); }
		}

		#endregion

		#region unused members

		[LCIDConversion(1)]
		[DispId(1095)]
		public void ActivateMicrosoftApp(Microsoft.Office.Interop.Excel.XlMSApplication Index)
		{
			throw new NotImplementedException();
		}

		[DispId(183)]
		public Microsoft.Office.Interop.Excel.Chart ActiveChart
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(815)]
		public Microsoft.Office.Interop.Excel.DialogSheet ActiveDialog
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(2394)]
		public int ActiveEncryptionSession
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(758)]
		public Microsoft.Office.Interop.Excel.MenuBar ActiveMenuBar
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(306)]
		public string ActivePrinter
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(759)]
		public Microsoft.Office.Interop.Excel.Window ActiveWindow
		{
			get { throw new NotImplementedException(); }
		}

		[LCIDConversion(3)]
		[DispId(216)]
		[TypeLibFunc(64)]
		public void AddChartAutoFormat(object Chart, string Name, [Optional]object Description)
		{
			throw new NotImplementedException();
		}

		[DispId(780)]
		[LCIDConversion(2)]
		public void AddCustomList(object ListArray, [Optional]object ByRow)
		{
			throw new NotImplementedException();
		}

		[DispId(549)]
		public Microsoft.Office.Interop.Excel.AddIns AddIns
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(930)]
		public bool AlertBeforeOverwriting
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(313)]
		public string AltStartupPath
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2381)]
		public bool AlwaysUseClearType
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1804)]
		public AnswerWizard AnswerWizard
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(148)]
		Microsoft.Office.Interop.Excel.Application Microsoft.Office.Interop.Excel._Application.Application
		{
			get { return this; }
		}

		[DispId(2254)]
		public bool ArbitraryXMLSupportAvailable
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(992)]
		public bool AskToUpdateLinks
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2386)]
		public IAssistance Assistance
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1438)]
		public Assistant Assistant
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1145)]
		public Microsoft.Office.Interop.Excel.AutoCorrect AutoCorrect
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1955)]
		public bool AutoFormatAsYouTypeReplaceHyperlinks
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1800)]
		public bool AutoPercentEntry
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1949)]
		public Microsoft.Office.Interop.Excel.AutoRecover AutoRecover
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1941)]
		public MsoAutomationSecurity AutomationSecurity
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(314)]
		public int Build
		{
			get { return 1; }
		}

		[DispId(1796)]
		public COMAddIns COMAddIns
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(315)]
		public bool CalculateBeforeSave
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1938)]
		public Microsoft.Office.Interop.Excel.XlCalculationInterruptKey CalculationInterruptKey
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1937)]
		public Microsoft.Office.Interop.Excel.XlCalculationState CalculationState
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1806)]
		public int CalculationVersion
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(318)]
		public bool CanPlaySounds
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(319)]
		public bool CanRecordSounds
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(139)]
		public string Caption
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(320)]
		public bool CellDragAndDrop
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(121)]
		public Microsoft.Office.Interop.Excel.Sheets Charts
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1952)]
		public void CheckAbort([Optional]object KeepAbort)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(3)]
		[DispId(505)]
		public bool CheckSpelling(string Word, [Optional]object CustomDictionary, [Optional]object IgnoreUppercase)
		{
			throw new NotImplementedException();
		}

		[DispId(365)]
		public bool ColorButtons
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1439)]
		public CommandBars CommandBars
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(323)]
		public Microsoft.Office.Interop.Excel.XlCommandUnderlines CommandUnderlines
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(324)]
		public bool ConstrainNumeric
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(233)]
		public bool ControlCharacters
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(325)]
		[LCIDConversion(5)]
		public dynamic ConvertFormula(object Formula, Microsoft.Office.Interop.Excel.XlReferenceStyle FromReferenceStyle, [Optional]object ToReferenceStyle, [Optional]object ToAbsolute, [Optional]object RelativeTo)
		{
			throw new NotImplementedException();
		}

		[DispId(991)]
		public bool CopyObjectsWithCells
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(149)]
		public Microsoft.Office.Interop.Excel.XlCreator Creator
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1161)]
		public Microsoft.Office.Interop.Excel.XlMousePointer Cursor
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(232)]
		public int CursorMovement
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(787)]
		public int CustomListCount
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(330)]
		public Microsoft.Office.Interop.Excel.XlCutCopyMode CutCopyMode
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(332)]
		public int DDEAppReturnCode
		{
			get { throw new NotImplementedException(); }
		}

		[LCIDConversion(2)]
		[DispId(333)]
		public void DDEExecute(int Channel, string String)
		{
			throw new NotImplementedException();
		}

		[DispId(334)]
		[LCIDConversion(2)]
		public int DDEInitiate(string App, string Topic)
		{
			throw new NotImplementedException();
		}

		[DispId(335)]
		[LCIDConversion(3)]
		public void DDEPoke(int Channel, object Item, object Data)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(2)]
		[DispId(336)]
		public dynamic DDERequest(int Channel, string Item)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(1)]
		[DispId(337)]
		public void DDETerminate(int Channel)
		{
			throw new NotImplementedException();
		}

		[DispId(331)]
		public int DataEntryMode
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1038)]
		public string DefaultFilePath
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1209)]
		public Microsoft.Office.Interop.Excel.XlFileFormat DefaultSaveFormat
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(229)]
		public int DefaultSheetDirection
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1797)]
		public Microsoft.Office.Interop.Excel.DefaultWebOptions DefaultWebOptions
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(2390)]
		public bool DeferAsyncQueries
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(217)]
		[LCIDConversion(1)]
		[TypeLibFunc(64)]
		public void DeleteChartAutoFormat(string Name)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(1)]
		[DispId(783)]
		public void DeleteCustomList(int ListNum)
		{
			throw new NotImplementedException();
		}

		[DispId(764)]
		public Microsoft.Office.Interop.Excel.Sheets DialogSheets
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(761)]
		public Microsoft.Office.Interop.Excel.Dialogs Dialogs
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(343)]
		public bool DisplayAlerts
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(322)]
		public bool DisplayClipboardWindow
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1196)]
		public Microsoft.Office.Interop.Excel.XlCommentDisplayMode DisplayCommentIndicator
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2251)]
		public bool DisplayDocumentActionTaskPane
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2380)]
		public bool DisplayDocumentInformationPanel
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(927)]
		public bool DisplayExcel4Menus
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2384)]
		public bool DisplayFormulaAutoComplete
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(344)]
		public bool DisplayFormulaBar
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1061)]
		public bool DisplayFullScreen
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1940)]
		public bool DisplayFunctionToolTips
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1213)]
		public bool DisplayInfoWindow
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1947)]
		public bool DisplayInsertOptions
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(345)]
		public bool DisplayNoteIndicator
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1946)]
		public bool DisplayPasteOptions
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(926)]
		public bool DisplayRecentFiles
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(346)]
		public bool DisplayScrollBars
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(347)]
		public bool DisplayStatusBar
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2252)]
		public void DisplayXMLSourcePane([Optional]object XmlMap)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(0)]
		[DispId(349)]
		public void DoubleClick()
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(64)]
		[DispId(1782)]
		public dynamic Dummy1([Optional]object Arg1, [Optional]object Arg2, [Optional]object Arg3, [Optional]object Arg4)
		{
			throw new NotImplementedException();
		}

		[DispId(1791)]
		[TypeLibFunc(64)]
		public bool Dummy10([Optional]object arg)
		{
			throw new NotImplementedException();
		}

		[DispId(1802)]
		public dynamic Dummy101
		{
			get { throw new NotImplementedException(); }
		}

		[TypeLibFunc(64)]
		[DispId(1792)]
		public void Dummy11()
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(64)]
		[DispId(1803)]
		public void Dummy12(Microsoft.Office.Interop.Excel.PivotTable p1, Microsoft.Office.Interop.Excel.PivotTable p2)
		{
			throw new NotImplementedException();
		}

		[DispId(1933)]
		[TypeLibFunc(64)]
		public dynamic Dummy13(object Arg1, [Optional]object Arg2, [Optional]object Arg3, [Optional]object Arg4, [Optional]object Arg5, [Optional]object Arg6, [Optional]object Arg7, [Optional]object Arg8, [Optional]object Arg9, [Optional]object Arg10, [Optional]object Arg11, [Optional]object Arg12, [Optional]object Arg13, [Optional]object Arg14, [Optional]object Arg15, [Optional]object Arg16, [Optional]object Arg17, [Optional]object Arg18, [Optional]object Arg19, [Optional]object Arg20, [Optional]object Arg21, [Optional]object Arg22, [Optional]object Arg23, [Optional]object Arg24, [Optional]object Arg25, [Optional]object Arg26, [Optional]object Arg27, [Optional]object Arg28, [Optional]object Arg29, [Optional]object Arg30)
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(64)]
		[DispId(1944)]
		public void Dummy14()
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(64)]
		[DispId(1783)]
		public dynamic Dummy2([Optional]object Arg1, [Optional]object Arg2, [Optional]object Arg3, [Optional]object Arg4, [Optional]object Arg5, [Optional]object Arg6, [Optional]object Arg7, [Optional]object Arg8)
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(64)]
		[DispId(2373)]
		public dynamic Dummy20(int grfCompareFunctions)
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(64)]
		[DispId(1784)]
		public dynamic Dummy3()
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(64)]
		[DispId(1785)]
		public dynamic Dummy4([Optional]object Arg1, [Optional]object Arg2, [Optional]object Arg3, [Optional]object Arg4, [Optional]object Arg5, [Optional]object Arg6, [Optional]object Arg7, [Optional]object Arg8, [Optional]object Arg9, [Optional]object Arg10, [Optional]object Arg11, [Optional]object Arg12, [Optional]object Arg13, [Optional]object Arg14, [Optional]object Arg15)
		{
			throw new NotImplementedException();
		}

		[DispId(1786)]
		[TypeLibFunc(64)]
		public dynamic Dummy5([Optional]object Arg1, [Optional]object Arg2, [Optional]object Arg3, [Optional]object Arg4, [Optional]object Arg5, [Optional]object Arg6, [Optional]object Arg7, [Optional]object Arg8, [Optional]object Arg9, [Optional]object Arg10, [Optional]object Arg11, [Optional]object Arg12, [Optional]object Arg13)
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(64)]
		[DispId(1787)]
		public dynamic Dummy6()
		{
			throw new NotImplementedException();
		}

		[DispId(1788)]
		[TypeLibFunc(64)]
		public dynamic Dummy7()
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(64)]
		[DispId(1789)]
		public dynamic Dummy8([Optional]object Arg1)
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(64)]
		[DispId(1790)]
		public dynamic Dummy9()
		{
			throw new NotImplementedException();
		}

		[DispId(929)]
		public bool EditDirectlyInCell
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1180)]
		public bool EnableAnimations
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1179)]
		public bool EnableAutoComplete
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1096)]
		public Microsoft.Office.Interop.Excel.XlEnableCancelKey EnableCancelKey
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1212)]
		public bool EnableEvents
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2388)]
		public bool EnableLargeOperationAlert
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2379)]
		public bool EnableLivePreview
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1197)]
		public bool EnableSound
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1064)]
		public bool EnableTipWizard
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1954)]
		public Microsoft.Office.Interop.Excel.ErrorCheckingOptions ErrorCheckingOptions
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(581)]
		public Microsoft.Office.Interop.Excel.Sheets Excel4IntlMacroSheets
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(579)]
		public Microsoft.Office.Interop.Excel.Sheets Excel4MacroSheets
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(350)]
		[LCIDConversion(1)]
		public dynamic ExecuteExcel4Macro(string String)
		{
			throw new NotImplementedException();
		}

		[DispId(1793)]
		public bool ExtendList
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1808)]
		public MsoFeatureInstall FeatureInstall
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1201)]
		public IFind FileFind
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1200)]
		public FileSearch FileSearch
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1201)]
		public bool FindFile()
		{
			throw new NotImplementedException();
		}

		[DispId(1934)]
		public Microsoft.Office.Interop.Excel.CellFormat FindFormat
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(351)]
		public bool FixedDecimal
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(352)]
		public int FixedDecimalPlaces
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2383)]
		public int FormulaBarHeight
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1948)]
		public bool GenerateGetPivotData
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2385)]
		public Microsoft.Office.Interop.Excel.XlGenerateTableRefs GenerateTableRefs
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[LCIDConversion(1)]
		[DispId(786)]
		public dynamic GetCustomListContents(int ListNum)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(1)]
		[DispId(785)]
		public int GetCustomListNum(object ListArray)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(5)]
		[DispId(1075)]
		public dynamic GetOpenFilename([Optional]object FileFilter, [Optional]object FilterIndex, [Optional]object Title, [Optional]object ButtonText, [Optional]object MultiSelect)
		{
			throw new NotImplementedException();
		}

		[DispId(1795)]
		public string GetPhonetic([Optional]object Text)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(5)]
		[DispId(1076)]
		public dynamic GetSaveAsFilename([Optional]object InitialFilename, [Optional]object FileFilter, [Optional]object FilterIndex, [Optional]object Title, [Optional]object ButtonText)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(2)]
		[DispId(475)]
		public void Goto([Optional]object Reference, [Optional]object Scroll)
		{
			throw new NotImplementedException();
		}

		[DispId(123)]
		public double Height
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(354)]
		[LCIDConversion(2)]
		public void Help([Optional]object HelpFile, [Optional]object HelpContextID)
		{
			throw new NotImplementedException();
		}

		[DispId(2395)]
		public bool HighQualityModeForGraphics
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1951)]
		public int Hinstance
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1950)]
		public int Hwnd
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(356)]
		public bool IgnoreRemoteRequests
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(357)]
		[LCIDConversion(8)]
		public dynamic InputBox(string Prompt, [Optional]object Title, [Optional]object Default, [Optional]object Left, [Optional]object Top, [Optional]object HelpFile, [Optional]object HelpContextID, [Optional]object Type)
		{
			throw new NotImplementedException();
		}

		[DispId(361)]
		public bool Interactive
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(766)]
		[LCIDConversion(30)]
		public Microsoft.Office.Interop.Excel.Range Intersect(Microsoft.Office.Interop.Excel.Range Arg1, Microsoft.Office.Interop.Excel.Range Arg2, [Optional]object Arg3, [Optional]object Arg4, [Optional]object Arg5, [Optional]object Arg6, [Optional]object Arg7, [Optional]object Arg8, [Optional]object Arg9, [Optional]object Arg10, [Optional]object Arg11, [Optional]object Arg12, [Optional]object Arg13, [Optional]object Arg14, [Optional]object Arg15, [Optional]object Arg16, [Optional]object Arg17, [Optional]object Arg18, [Optional]object Arg19, [Optional]object Arg20, [Optional]object Arg21, [Optional]object Arg22, [Optional]object Arg23, [Optional]object Arg24, [Optional]object Arg25, [Optional]object Arg26, [Optional]object Arg27, [Optional]object Arg28, [Optional]object Arg29, [Optional]object Arg30)
		{
			throw new NotImplementedException();
		}

		[DispId(363)]
		public bool Iteration
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1801)]
		public LanguageSettings LanguageSettings
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(364)]
		public bool LargeButtons
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2389)]
		public int LargeOperationCellThousandCount
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(127)]
		public double Left
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(366)]
		public string LibraryPath
		{
			get { throw new NotImplementedException(); }
		}

		[LCIDConversion(10)]
		[DispId(1135)]
		public void MacroOptions([Optional]object Macro, [Optional]object Description, [Optional]object HasMenu, [Optional]object MenuText, [Optional]object HasShortcutKey, [Optional]object ShortcutKey, [Optional]object Category, [Optional]object StatusBar, [Optional]object HelpContextID, [Optional]object HelpFile)
		{
			throw new NotImplementedException();
		}

		[DispId(942)]
		public dynamic MailSession
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(971)]
		public Microsoft.Office.Interop.Excel.XlMailSystem MailSystem
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1959)]
		public bool MapPaperSize
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(367)]
		public bool MathCoprocessorAvailable
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(368)]
		public double MaxChange
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(369)]
		public int MaxIterations
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(370)]
		public int MemoryFree
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(371)]
		public int MemoryTotal
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(372)]
		public int MemoryUsed
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(589)]
		public Microsoft.Office.Interop.Excel.MenuBars MenuBars
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(582)]
		public Microsoft.Office.Interop.Excel.Modules Modules
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(373)]
		public bool MouseAvailable
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(374)]
		public bool MoveAfterReturn
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1144)]
		public Microsoft.Office.Interop.Excel.XlDirection MoveAfterReturnDirection
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2391)]
		public Microsoft.Office.Interop.Excel.MultiThreadedCalculation MultiThreadedCalculation
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(110)]
		public string Name
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(442)]
		public Microsoft.Office.Interop.Excel.Names Names
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(388)]
		public string NetworkTemplatesPath
		{
			get { throw new NotImplementedException(); }
		}

		[LCIDConversion(0)]
		[DispId(972)]
		public Microsoft.Office.Interop.Excel.Workbook NextLetter()
		{
			throw new NotImplementedException();
		}

		[DispId(1203)]
		public Microsoft.Office.Interop.Excel.ODBCErrors ODBCErrors
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1204)]
		public int ODBCTimeout
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1794)]
		public Microsoft.Office.Interop.Excel.OLEDBErrors OLEDBErrors
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(625)]
		public string OnCalculate
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(629)]
		public string OnData
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(628)]
		public string OnDoubleClick
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(627)]
		public string OnEntry
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(626)]
		[LCIDConversion(2)]
		public void OnKey(string Key, [Optional]object Procedure)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(2)]
		[DispId(769)]
		public void OnRepeat(string Text, string Procedure)
		{
			throw new NotImplementedException();
		}

		[DispId(1031)]
		public string OnSheetActivate
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1081)]
		public string OnSheetDeactivate
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(624)]
		[LCIDConversion(4)]
		public void OnTime(object EarliestTime, string Procedure, [Optional]object LatestTime, [Optional]object Schedule)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(2)]
		[DispId(770)]
		public void OnUndo(string Text, string Procedure)
		{
			throw new NotImplementedException();
		}

		[DispId(623)]
		public string OnWindow
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(375)]
		public string OperatingSystem
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(376)]
		public string OrganizationName
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(150)]
		public Microsoft.Office.Interop.Excel.Application Parent
		{
			get { return null; }
		}

		[DispId(291)]
		public string Path
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(377)]
		public string PathSeparator
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1205)]
		public bool PivotTableSelection
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1798)]
		public string ProductCode
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1062)]
		public bool PromptForSummaryInfo
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1963)]
		public Microsoft.Office.Interop.Excel.RTD RTD
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1932)]
		public bool Ready
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1202)]
		public Microsoft.Office.Interop.Excel.RecentFiles RecentFiles
		{
			get { throw new NotImplementedException(); }
		}

		[LCIDConversion(2)]
		[DispId(773)]
		public void RecordMacro([Optional]object BasicCode, [Optional]object XlmCode)
		{
			throw new NotImplementedException();
		}

		[DispId(379)]
		public bool RecordRelative
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(380)]
		public Microsoft.Office.Interop.Excel.XlReferenceStyle ReferenceStyle
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(30)]
		[LCIDConversion(1)]
		public bool RegisterXLL(string Filename)
		{
			throw new NotImplementedException();
		}

		[DispId(301)]
		[LCIDConversion(0)]
		public void Repeat()
		{
			throw new NotImplementedException();
		}

		[DispId(1935)]
		public Microsoft.Office.Interop.Excel.CellFormat ReplaceFormat
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[LCIDConversion(0)]
		[TypeLibFunc(64)]
		[DispId(928)]
		public void ResetTipWizard()
		{
			throw new NotImplementedException();
		}

		[DispId(1206)]
		public bool RollZoom
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(259)]
		public dynamic Run([Optional]object Macro, [Optional]object Arg1, [Optional]object Arg2, [Optional]object Arg3, [Optional]object Arg4, [Optional]object Arg5, [Optional]object Arg6, [Optional]object Arg7, [Optional]object Arg8, [Optional]object Arg9, [Optional]object Arg10, [Optional]object Arg11, [Optional]object Arg12, [Optional]object Arg13, [Optional]object Arg14, [Optional]object Arg15, [Optional]object Arg16, [Optional]object Arg17, [Optional]object Arg18, [Optional]object Arg19, [Optional]object Arg20, [Optional]object Arg21, [Optional]object Arg22, [Optional]object Arg23, [Optional]object Arg24, [Optional]object Arg25, [Optional]object Arg26, [Optional]object Arg27, [Optional]object Arg28, [Optional]object Arg29, [Optional]object Arg30)
		{
			throw new NotImplementedException();
		}

		[DispId(382)]
		public bool ScreenUpdating
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(383)]
		[LCIDConversion(2)]
		public void SendKeys(object Keys, [Optional]object Wait)
		{
			throw new NotImplementedException();
		}

		[DispId(219)]
		[TypeLibFunc(64)]
		public void SetDefaultChart([Optional]object FormatName, [Optional]object Gallery)
		{
			throw new NotImplementedException();
		}

		[DispId(2392)]
		public int SharePointVersion(string bstrUrl)
		{
			throw new NotImplementedException();
		}

		[DispId(993)]
		public int SheetsInNewWorkbook
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1207)]
		public bool ShowChartTipNames
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1208)]
		public bool ShowChartTipValues
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2378)]
		public bool ShowDevTools
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2377)]
		public bool ShowMenuFloaties
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2376)]
		public bool ShowSelectionFloaties
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1960)]
		public bool ShowStartupDialog
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(387)]
		public bool ShowToolTips
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1807)]
		public bool ShowWindowsInTaskbar
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1956)]
		public Microsoft.Office.Interop.Excel.SmartTagRecognizers SmartTagRecognizers
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1958)]
		public Microsoft.Office.Interop.Excel.Speech Speech
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1957)]
		public Microsoft.Office.Interop.Excel.SpellingOptions SpellingOptions
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(385)]
		public string StartupPath
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(386)]
		public dynamic StatusBar
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[TypeLibFunc(64)]
		[DispId(2255)]
		public dynamic Support(object Object, int ID, [Optional]object arg)
		{
			throw new NotImplementedException();
		}

		[DispId(381)]
		public string TemplatesPath
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(552)]
		public Microsoft.Office.Interop.Excel.Toolbars Toolbars
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(126)]
		public double Top
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(310)]
		public string TransitionMenuKey
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(311)]
		public int TransitionMenuKeyAction
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(312)]
		public bool TransitionNavigKeys
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2)]
		public int UILanguage
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[LCIDConversion(0)]
		[DispId(303)]
		public void Undo()
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(30)]
		[DispId(779)]
		public Microsoft.Office.Interop.Excel.Range Union(Microsoft.Office.Interop.Excel.Range Arg1, Microsoft.Office.Interop.Excel.Range Arg2, [Optional]object Arg3, [Optional]object Arg4, [Optional]object Arg5, [Optional]object Arg6, [Optional]object Arg7, [Optional]object Arg8, [Optional]object Arg9, [Optional]object Arg10, [Optional]object Arg11, [Optional]object Arg12, [Optional]object Arg13, [Optional]object Arg14, [Optional]object Arg15, [Optional]object Arg16, [Optional]object Arg17, [Optional]object Arg18, [Optional]object Arg19, [Optional]object Arg20, [Optional]object Arg21, [Optional]object Arg22, [Optional]object Arg23, [Optional]object Arg24, [Optional]object Arg25, [Optional]object Arg26, [Optional]object Arg27, [Optional]object Arg28, [Optional]object Arg29, [Optional]object Arg30)
		{
			throw new NotImplementedException();
		}

		[DispId(389)]
		public double UsableHeight
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(390)]
		public double UsableWidth
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1961)]
		public bool UseSystemSeparators
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1936)]
		public Microsoft.Office.Interop.Excel.UsedObjects UsedObjects
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1210)]
		public bool UserControl
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1799)]
		public string UserLibraryPath
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(391)]
		public string UserName
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1211)]
		public Microsoft.Vbe.Interop.VBE VBE
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(6)]
		public string Value
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(392)]
		public string Version
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(558)]
		public bool Visible { get; set; }

		[LCIDConversion(1)]
		[DispId(788)]
		public void Volatile([Optional]object Volatile)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(1)]
		[DispId(1770)]
		public bool Wait(object Time)
		{
			throw new NotImplementedException();
		}

		[DispId(2382)]
		public bool WarnOnFunctionNameConflict
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1939)]
		public Microsoft.Office.Interop.Excel.Watches Watches
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(122)]
		public double Width
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(396)]
		public Microsoft.Office.Interop.Excel.XlWindowState WindowState
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(430)]
		public Microsoft.Office.Interop.Excel.Windows Windows
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(395)]
		public bool WindowsForPens
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1440)]
		public Microsoft.Office.Interop.Excel.WorksheetFunction WorksheetFunction
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(0)]
		public string _Default
		{
			get { throw new NotImplementedException(); }
		}

		[TypeLibFunc(1024)]
		[DispId(-5)]
		[LCIDConversion(1)]
		public dynamic _Evaluate(object Name)
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(1088)]
		[LCIDConversion(0)]
		[DispId(1068)]
		public void _FindFile()
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(1024)]
		[DispId(806)]
		[LCIDConversion(31)]
		public dynamic _Run2([Optional]object Macro, [Optional]object Arg1, [Optional]object Arg2, [Optional]object Arg3, [Optional]object Arg4, [Optional]object Arg5, [Optional]object Arg6, [Optional]object Arg7, [Optional]object Arg8, [Optional]object Arg9, [Optional]object Arg10, [Optional]object Arg11, [Optional]object Arg12, [Optional]object Arg13, [Optional]object Arg14, [Optional]object Arg15, [Optional]object Arg16, [Optional]object Arg17, [Optional]object Arg18, [Optional]object Arg19, [Optional]object Arg20, [Optional]object Arg21, [Optional]object Arg22, [Optional]object Arg23, [Optional]object Arg24, [Optional]object Arg25, [Optional]object Arg26, [Optional]object Arg27, [Optional]object Arg28, [Optional]object Arg29, [Optional]object Arg30)
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(1088)]
		[LCIDConversion(30)]
		[DispId(169)]
		public dynamic _WSFunction([Optional]object Arg1, [Optional]object Arg2, [Optional]object Arg3, [Optional]object Arg4, [Optional]object Arg5, [Optional]object Arg6, [Optional]object Arg7, [Optional]object Arg8, [Optional]object Arg9, [Optional]object Arg10, [Optional]object Arg11, [Optional]object Arg12, [Optional]object Arg13, [Optional]object Arg14, [Optional]object Arg15, [Optional]object Arg16, [Optional]object Arg17, [Optional]object Arg18, [Optional]object Arg19, [Optional]object Arg20, [Optional]object Arg21, [Optional]object Arg22, [Optional]object Arg23, [Optional]object Arg24, [Optional]object Arg25, [Optional]object Arg26, [Optional]object Arg27, [Optional]object Arg28, [Optional]object Arg29, [Optional]object Arg30)
		{
			throw new NotImplementedException();
		}

		[TypeLibFunc(1088)]
		[DispId(393)]
		[LCIDConversion(1)]
		public void _Wait(object Time)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(1)]
		[DispId(317)]
		public dynamic get_Caller([Optional]object Index)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(1)]
		[DispId(321)]
		public dynamic get_ClipboardFormats([Optional]object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(931)]
		[LCIDConversion(2)]
		public dynamic get_FileConverters([Optional]object Index1, [Optional]object Index2)
		{
			throw new NotImplementedException();
		}

		[DispId(1942)]
		public FileDialog get_FileDialog(MsoFileDialogType fileDialogType)
		{
			throw new NotImplementedException();
		}

		[DispId(362)]
		[LCIDConversion(1)]
		public dynamic get_International([Optional]object Index)
		{
			throw new NotImplementedException();
		}

		[DispId(378)]
		[LCIDConversion(1)]
		public dynamic get_PreviousSelections([Optional]object Index)
		{
			throw new NotImplementedException();
		}

		[LCIDConversion(2)]
		[DispId(775)]
		public dynamic get_RegisteredFunctions([Optional]object Index1, [Optional]object Index2)
		{
			throw new NotImplementedException();
		}

		[DispId(776)]
		[TypeLibFunc(64)]
		public Microsoft.Office.Interop.Excel.Menu get_ShortcutMenus(int Index)
		{
			throw new NotImplementedException();
		}

		/////////////////////////////////////////////////////////////////////////////////////

		public event Microsoft.Office.Interop.Excel.AppEvents_AfterCalculateEventHandler AfterCalculate;

		event Microsoft.Office.Interop.Excel.AppEvents_NewWorkbookEventHandler Microsoft.Office.Interop.Excel.AppEvents_Event.NewWorkbook
		{
			add { throw new NotImplementedException(); }
			remove { throw new NotImplementedException(); }
		}

		public event Microsoft.Office.Interop.Excel.AppEvents_SheetActivateEventHandler SheetActivate;

		public event Microsoft.Office.Interop.Excel.AppEvents_SheetBeforeDoubleClickEventHandler SheetBeforeDoubleClick;

		public event Microsoft.Office.Interop.Excel.AppEvents_SheetBeforeRightClickEventHandler SheetBeforeRightClick;

		public event Microsoft.Office.Interop.Excel.AppEvents_SheetCalculateEventHandler SheetCalculate;

		public event Microsoft.Office.Interop.Excel.AppEvents_SheetChangeEventHandler SheetChange;

		public event Microsoft.Office.Interop.Excel.AppEvents_SheetDeactivateEventHandler SheetDeactivate;

		public event Microsoft.Office.Interop.Excel.AppEvents_SheetFollowHyperlinkEventHandler SheetFollowHyperlink;

		public event Microsoft.Office.Interop.Excel.AppEvents_SheetPivotTableUpdateEventHandler SheetPivotTableUpdate;

		public event Microsoft.Office.Interop.Excel.AppEvents_SheetSelectionChangeEventHandler SheetSelectionChange;

		public event Microsoft.Office.Interop.Excel.AppEvents_WindowActivateEventHandler WindowActivate;

		public event Microsoft.Office.Interop.Excel.AppEvents_WindowDeactivateEventHandler WindowDeactivate;

		public event Microsoft.Office.Interop.Excel.AppEvents_WindowResizeEventHandler WindowResize;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookActivateEventHandler WorkbookActivate;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookAddinInstallEventHandler WorkbookAddinInstall;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookAddinUninstallEventHandler WorkbookAddinUninstall;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookAfterXmlExportEventHandler WorkbookAfterXmlExport;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookAfterXmlImportEventHandler WorkbookAfterXmlImport;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookBeforeCloseEventHandler WorkbookBeforeClose;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookBeforePrintEventHandler WorkbookBeforePrint;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookBeforeSaveEventHandler WorkbookBeforeSave;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookBeforeXmlExportEventHandler WorkbookBeforeXmlExport;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookBeforeXmlImportEventHandler WorkbookBeforeXmlImport;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookDeactivateEventHandler WorkbookDeactivate;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookNewSheetEventHandler WorkbookNewSheet;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookOpenEventHandler WorkbookOpen;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookPivotTableCloseConnectionEventHandler WorkbookPivotTableCloseConnection;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookPivotTableOpenConnectionEventHandler WorkbookPivotTableOpenConnection;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookRowsetCompleteEventHandler WorkbookRowsetComplete;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookSyncEventHandler WorkbookSync;

		#endregion

		#region excel version specific members
		/**/

		public Microsoft.Office.Interop.Excel.FileExportConverters FileExportConverters
		{
			get { throw new NotImplementedException(); }
		}

		public Microsoft.Office.Interop.Excel.ProtectedViewWindow ActiveProtectedViewWindow
		{
			get { throw new NotImplementedException(); }
		}

		public Microsoft.Office.Interop.Excel.AddIns2 AddIns2
		{
			get { throw new NotImplementedException(); }
		}

		public bool ChartDataPointTrack
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public string ClusterConnector
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool Dummy22
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool Dummy23
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool EnableCheckFileExtensions
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool EnableMacroAnimations
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public MsoFileValidationMode FileValidation
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public Microsoft.Office.Interop.Excel.XlFileValidationPivotMode FileValidationPivot
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool FlashFill
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool FlashFillMode
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic HinstancePtr
		{
			get { throw new NotImplementedException(); }
		}

		public bool IsSandboxed
		{
			get { throw new NotImplementedException(); }
		}

		public void MacroOptions2([OptionalAttribute]object Macro, [OptionalAttribute]object Description, [OptionalAttribute]object HasMenu, [OptionalAttribute]object MenuText, [OptionalAttribute]object HasShortcutKey, [OptionalAttribute]object ShortcutKey, [OptionalAttribute]object Category, [OptionalAttribute]object StatusBar, [OptionalAttribute]object HelpContextID, [OptionalAttribute]object HelpFile, [OptionalAttribute]object ArgumentDescriptions)
		{
			throw new NotImplementedException();
		}

		public bool MergeInstances
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool PrintCommunication
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public Microsoft.Office.Interop.Excel.ProtectedViewWindows ProtectedViewWindows
		{
			get { throw new NotImplementedException(); }
		}

		/*public Microsoft.Office.Interop.Excel.QuickAnalysis QuickAnalysis
		{
			get { throw new NotImplementedException(); }
		}*/

		public bool Quitting
		{
			get { throw new NotImplementedException(); }
		}

		public bool SaveISO8601Dates
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public bool ShowQuickAnalysis
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public SmartArtColors SmartArtColors
		{
			get { throw new NotImplementedException(); }
		}

		public SmartArtLayouts SmartArtLayouts
		{
			get { throw new NotImplementedException(); }
		}

		public SmartArtQuickStyles SmartArtQuickStyles
		{
			get { throw new NotImplementedException(); }
		}

		public bool UseClusterConnector
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}


		public event Microsoft.Office.Interop.Excel.AppEvents_ProtectedViewWindowActivateEventHandler ProtectedViewWindowActivate;

		public event Microsoft.Office.Interop.Excel.AppEvents_ProtectedViewWindowBeforeCloseEventHandler ProtectedViewWindowBeforeClose;

		public event Microsoft.Office.Interop.Excel.AppEvents_ProtectedViewWindowBeforeEditEventHandler ProtectedViewWindowBeforeEdit;

		public event Microsoft.Office.Interop.Excel.AppEvents_ProtectedViewWindowDeactivateEventHandler ProtectedViewWindowDeactivate;

		public event Microsoft.Office.Interop.Excel.AppEvents_ProtectedViewWindowOpenEventHandler ProtectedViewWindowOpen;

		public event Microsoft.Office.Interop.Excel.AppEvents_ProtectedViewWindowResizeEventHandler ProtectedViewWindowResize;

		//public event Microsoft.Office.Interop.Excel.AppEvents_SheetBeforeDeleteEventHandler SheetBeforeDelete;

		//public event Microsoft.Office.Interop.Excel.AppEvents_SheetLensGalleryRenderCompleteEventHandler SheetLensGalleryRenderComplete;

		public event Microsoft.Office.Interop.Excel.AppEvents_SheetPivotTableAfterValueChangeEventHandler SheetPivotTableAfterValueChange;

		public event Microsoft.Office.Interop.Excel.AppEvents_SheetPivotTableBeforeAllocateChangesEventHandler SheetPivotTableBeforeAllocateChanges;

		public event Microsoft.Office.Interop.Excel.AppEvents_SheetPivotTableBeforeCommitChangesEventHandler SheetPivotTableBeforeCommitChanges;

		public event Microsoft.Office.Interop.Excel.AppEvents_SheetPivotTableBeforeDiscardChangesEventHandler SheetPivotTableBeforeDiscardChanges;

		//public event Microsoft.Office.Interop.Excel.AppEvents_SheetTableUpdateEventHandler SheetTableUpdate;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookAfterSaveEventHandler WorkbookAfterSave;

		//public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookModelChangeEventHandler WorkbookModelChange;

		public event Microsoft.Office.Interop.Excel.AppEvents_WorkbookNewChartEventHandler WorkbookNewChart;

		/*//**/
		#endregion
	}
}
