﻿using System;
using System.Threading;
using Google.Apis.Drive.v2;
using Google.Apis.Auth.OAuth2;
using Google.Apis.Script.v1;
using Google.Apis.Script.v1.Data;
using Google.Apis.Services;
using Google.Apis.Util.Store;
using Google.GData.Client;
using Google.GData.Spreadsheets;


namespace GSpreadCOM.Domain
{
	class ApplicationService
	{
		public const string SCOPE_SPREADSHEETS = "https://spreadsheets.google.com/feeds";
		public const string REDIRECT_URI = "urn:ietf:wg:oauth:2.0:oob";

		static string[] Scopes = new string[] {
		    DriveService.Scope.Drive,                   // view and manage your files and documents
		    DriveService.Scope.DriveAppdata,            // view and manage its own configuration data
		    DriveService.Scope.DriveAppsReadonly,       // view your drive apps
		    DriveService.Scope.DriveFile,               // view and manage files created by this app
		    DriveService.Scope.DriveMetadataReadonly,   // view metadata for files
		    DriveService.Scope.DriveReadonly,           // view files and documents on your drive
		    DriveService.Scope.DriveScripts,            // modify your app scripts
		    ScriptService.Scope.Drive,

		    SCOPE_SPREADSHEETS,

		    "https://www.googleapis.com/auth/spreadsheets",
		    "https://docs.google.com/feeds"
		};

		public DocumentsService Documents { get; private set; }
		public DriveService Drive { get; private set; }
		public AppsScriptService Script { get; private set; }
		public SpreadsheetsService Spreadsheets { get; private set; }

		private UserCredential credential;

		public ApplicationService()
		{
			GoogleServicesAuthorization();

			Documents = new DocumentsService(this);
		}

		public void GoogleServicesAuthorization(string user = "", string clientId = "", string clientSecret = "", string scriptId = "")
		{
			if (String.IsNullOrWhiteSpace(clientId))
			{
				clientId = Config.GetAttribute("client", "id");
				if (String.IsNullOrWhiteSpace(clientId))
				{
					throw new ArgumentException("", "clientId");
				}
			}

			if (String.IsNullOrWhiteSpace(clientSecret))
			{
				clientSecret = Config.GetAttribute("client", "secret");
				if (String.IsNullOrWhiteSpace(clientSecret))
				{
					throw new ArgumentException("", "clientSecret");
				}
			}

			if (String.IsNullOrWhiteSpace(user))
			{
				user = Config.GetAttribute("user", "name");
				if (String.IsNullOrWhiteSpace(user))
				{
					user = Guid.NewGuid().ToString();
				}
			}

			this.credential = GoogleWebAuthorizationBroker.AuthorizeAsync(
				new ClientSecrets { ClientId = clientId, ClientSecret = clientSecret },
				Scopes,
				user,
				CancellationToken.None
			).Result;

			var initializer = new BaseClientService.Initializer {
				HttpClientInitializer = this.credential,
				ApplicationName = Config.APP_NAME,
			};

			Drive = new DriveService(initializer);

			Script = new AppsScriptService(new ScriptService(initializer), scriptId);

			OAuth2Parameters parameters = new OAuth2Parameters()
			{
				ClientId = clientId,
				ClientSecret = clientSecret,
				RedirectUri = REDIRECT_URI,
				Scope = SCOPE_SPREADSHEETS,
				AccessToken = this.credential.Token.AccessToken,
				RefreshToken = this.credential.Token.RefreshToken
			};

			Spreadsheets = new SpreadsheetsService(Config.APP_NAME)
			{
				RequestFactory = new GOAuth2RequestFactory(null, Config.APP_NAME, parameters)
			};
		}

		public void RevokeAuthorizationToken()
		{
			if (this.credential != null)
			{
				this.credential.RevokeTokenAsync(CancellationToken.None);
			}
		}
	}
}
