﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using GSpreadCOM.Domain;

namespace GSpreadCOM
{
	[TypeLibType(TypeLibTypeFlags.FCanCreate | TypeLibTypeFlags.FDual | TypeLibTypeFlags.FDispatchable)]
	[ClassInterface(ClassInterfaceType.AutoDual)]
	[Guid("57E6845F-9402-4EAC-9508-3A3082617DAC")]
	[ComVisible(true)]
	public class Font : Microsoft.Office.Interop.Excel.Font
	{
		protected const string VARIES = "varies";
		protected const string NORMAL = "normal";
		protected const string BOLD = "bold";
		protected const string ITALIC = "italic";
		protected const string NONE = "none";
		protected const string UNDERLINE = "underline";
		protected const string LINE_THROUGH = "line-through";

		protected const int FAULT_SIZE = -1;
		protected const string DEFAULT_COLOR = "black";
		protected const int DEFAULT_SIZE = 10;

		protected Range range;

		public Font(Range range)
		{
			this.range = range;
		}

		/// <summary>
		/// Returns an Application object that represents the creator of the specified object. Read-only.
		/// </summary>
		public Microsoft.Office.Interop.Excel.Application Application
		{
			get { return range.Application; }
		}

		/// <summary>
		/// True if the font is bold. Read/write Boolean.
		/// </summary>
		public dynamic Bold
		{
			get
			{
				return range.ScriptService.call("getFontWeight", range) == BOLD;
			}
			set
			{
				if (value is bool) {
					range.ScriptService.call("setFontWeight", range, (value) ? BOLD : NORMAL);
				}
			}
		}

		/// <summary>
		/// Returns or sets the primary color of the object. Read/write.
		/// </summary>
		public dynamic Color
		{
			get
			{
				string htmlColor = GetFontColor();
				if (htmlColor == null || htmlColor == VARIES)
				{
					return ExcelColor.FAULT_VALUE;
				}
				return new ExcelColor(htmlColor).IntValue;
			}
			set
			{
				SetFontColor(new ExcelColor(value).HtmlValue);
			}
		}

		/// <summary>
		/// Returns or sets an int value that represents the color table index of the font.
		/// </summary>
		public dynamic ColorIndex
		{
			get
			{
				string htmlColor = GetFontColor();
				if (htmlColor == null || htmlColor == VARIES)
				{
					return ExcelColor.FAULT_VALUE;
				}
				return ExcelColorPalette.GetColorIndex(htmlColor);
			}
			set
			{
				SetFontColor(ExcelColorPalette.GetByIndex(value).HtmlValue);
			}
		}

		/// <summary>
		/// True if the font style is italic. Read/write Boolean.
		/// </summary>
		public dynamic Italic
		{
			get
			{
				return range.ScriptService.call("getFontStyle", range) == ITALIC;
			}
			set
			{
				if (value is bool) {
					range.ScriptService.call("setFontStyle", range, (value) ? ITALIC : NORMAL);
				}
			}
		}

		/// <summary>
		/// Returns or sets the size of the font. Read/write.
		/// </summary>
		public dynamic Size
		{
			get
			{
				string sizeStr = range.ScriptService.call("getFontSize", range).ToString();
				int size;
				if (!Int32.TryParse(sizeStr, out size))
				{
					return FAULT_SIZE;
				}

				return size;
			}
			set
			{
				if ((value is int || value is long) && value > 0)
				{
					range.ScriptService.call("setFontSize", range, value);
				}
			}
		}

		/// <summary>
		/// True if the font is struck through with a horizontal line. Read/write Boolean.
		/// </summary>
		public dynamic Strikethrough
		{
			get
			{
				return GetFontLine() == LINE_THROUGH;
			}
			set
			{
				if (value is bool) {
					if (value)
					{
						SetFontLine(LINE_THROUGH);
					}
					else
					{
						string line = GetFontLine();
						if (line == LINE_THROUGH || line == VARIES)
						{
							SetFontLine(NONE);
						}
					}
				}
			}
		}

		/// <summary>
		/// Returns or sets the type of underline applied to the font. Read/write XlUnderlineStyle.
		/// </summary>
		public dynamic Underline
		{
			get
			{
				return (GetFontLine() == UNDERLINE) ? Microsoft.Office.Interop.Excel.XlUnderlineStyle.xlUnderlineStyleSingle : Microsoft.Office.Interop.Excel.XlUnderlineStyle.xlUnderlineStyleNone;
			}
			set
			{
				if (value is Microsoft.Office.Interop.Excel.XlUnderlineStyle)
				{
					if (value == Microsoft.Office.Interop.Excel.XlUnderlineStyle.xlUnderlineStyleNone)
					{
						string line = GetFontLine();
						if (line == UNDERLINE || line == VARIES)
						{
							SetFontLine(NONE);
						}
					}
					else
					{
						SetFontLine(UNDERLINE);
					}
				}
			}
		}

		#region service members

		public void Reset()
		{
			Color = DEFAULT_COLOR;
			Size = DEFAULT_SIZE;
			Bold = false;
			Italic = false;
			Underline = Microsoft.Office.Interop.Excel.XlUnderlineStyle.xlUnderlineStyleNone;
		}

		protected string GetFontColor()
		{
			return range.ScriptService.call("getFontColor", range);
		}

		protected void SetFontColor(string htmlColor)
		{
			range.ScriptService.call("setFontColor", range, htmlColor);
		}

		protected string GetFontLine()
		{
			return range.ScriptService.call("getFontLine", range);
		}

		protected void SetFontLine(string line)
		{
			range.ScriptService.call("setFontLine", range, line);
		}

		#endregion

		#region unused members

		public dynamic Background
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public Microsoft.Office.Interop.Excel.XlCreator Creator
		{
			get { throw new NotImplementedException(); }
		}

		public dynamic FontStyle
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic Name
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic OutlineFont
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic Parent
		{
			get { throw new NotImplementedException(); }
		}

		public dynamic Shadow
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic Subscript
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic Superscript
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic ThemeColor
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public Microsoft.Office.Interop.Excel.XlThemeFont ThemeFont
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		public dynamic TintAndShade
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		#endregion
	}
}
