﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using Google.GData.Spreadsheets;
using Google.GData.Client;
using System.Net;
using System.IO;
using Newtonsoft.Json.Linq;
using GSpreadCOM.Domain;

namespace GSpreadCOM
{
	[TypeLibType(TypeLibTypeFlags.FCanCreate | TypeLibTypeFlags.FDual | TypeLibTypeFlags.FDispatchable)]
	//[ClassInterface(ClassInterfaceType.AutoDual)]
	[Guid("D1BD92B5-343E-4504-8B15-0AE49B5BAD80")]
	[ComVisible(true)]
	public class Range : Microsoft.Office.Interop.Excel.Range
	{
		protected enum RangeType
		{
			Cells,
			Columns,
			Rows
		}

		internal Document Document { get; private set; }
		internal Sheet Sheet { get; private set; }
		internal CellService Service { get; private set; }
		internal AppsScriptService ScriptService { get; private set; }
		
		protected const int LastRow = 1048576;
		protected const int LastColumn = 16384;

		protected RangeType type;
		protected int fromRow = 1;
		protected int toRow = Range.LastRow;
		protected int fromColumn = 1;
		protected int toColumn = Range.LastColumn;
		protected Style style;

		public Range(Worksheet worksheet)
		{
			Initialize();

			this.Worksheet = worksheet;
			this.Sheet = worksheet.Sheet;
			this.Service = Sheet.Cells;
			this.Document = Sheet.Service.Document;
			this.ScriptService = Document.Documents.Application.Script;

			this.type = RangeType.Cells;
		}

		public Range(Range parent, int fromRow, int toRow, int fromColumn, int toColumn)
		{
			Initialize(parent);

			this.fromRow = Math.Min(fromRow, toRow);
			this.toRow = Math.Max(fromRow, toRow);
			this.fromColumn = Math.Min(fromColumn, toColumn);
			this.toColumn = Math.Max(fromColumn, toColumn);

			Normalize();
		}

		protected Range(Range parent, RangeType type)
		{
			Initialize(parent);

			this.type = type;
		}

		protected void Initialize(Range parent = null) {
			if (parent != null) {
				this.Worksheet = parent.Worksheet;
				this.Service = parent.Service;
				this.Sheet = parent.Sheet;
				this.Document = parent.Document;
				this.ScriptService = parent.ScriptService;

				this.fromRow = parent.fromRow;
				this.fromColumn = parent.fromColumn;
				this.toRow = parent.toRow;
				this.toColumn = parent.toColumn;

				this.type = parent.type;
			}
			this.style = new Style(this);
		}

		#region properties

		internal int FromRow
		{
			get { return fromRow; }
		}

		internal int ToRow
		{
			get { return toRow; }
		}

		internal int FromColumn
		{
			get { return fromColumn; }
		}

		internal int ToColumn
		{
			get { return toColumn; }
		}

		/// <summary>
		/// Returns an Application object that represents the creator of the specified object. Read-only.
		/// </summary>
		[DispId(148)]
		public Microsoft.Office.Interop.Excel.Application Application
		{
			get { return Worksheet.Application; }
		}

		/// <summary>
		/// Returns a Range object that represents a range at an offset to the specified range.
		/// </summary>
		/// <param name="RowIndex"></param>
		/// <param name="ColumnIndex"></param>
		/// <returns></returns>
		[DispId(0)]
		public Microsoft.Office.Interop.Excel.Range this[[Optional]int RowIndex, [Optional]int ColumnIndex]
		{
			get
			{
				return this;
			}
		}

		/// <summary>
		/// Returns a Range object that represents a range at an offset to the specified range.
		/// </summary>
		/// <param name="RowIndex"></param>
		/// <param name="ColumnIndex"></param>
		/// <returns></returns>
		[DispId(0)]
		public dynamic this[object RowIndex, [Optional]object ColumnIndex]
		{
			get
			{
				int fromR = 0;
				int toR = 0;
				int fromC = 0;
				int toC = 0;

				bool rowIsNum = Int32.TryParse(RowIndex.ToString(), out fromR);
				bool columnIsNum = Int32.TryParse(ColumnIndex.ToString(), out fromC);

				if (rowIsNum && columnIsNum)
				{
					// single cell

					toR = fromR;
					toC = fromC;
				}
				else if (rowIsNum)
				{
					switch (type)
					{
					case RangeType.Cells:
						// single cell by its number in table

						int columns = toColumn - fromColumn + 1;
						int number = fromR;
						fromR = number / columns + 1;
						fromC = number % columns;
						toR = fromR;
						toC = fromC;

						break;
					case RangeType.Columns:
						// single column

						fromC = fromR;
						toC = fromC;
						fromR = 1;
						toR = toRow - fromRow + 1;

						break;
					case RangeType.Rows:
						// single row

						toR = fromR;
						fromC = 1;
						toC = toColumn - fromColumn + 1;

						break;
					}
				}
				else
				{
					throw new InvalidOperationException();
				}

				if (toR != Range.LastRow)
				{
					fromR += fromRow - 1;
					toR += fromRow - 1;
				}
				if (toC != Range.LastColumn)
				{
					fromC += fromColumn - 1;
					toC += fromColumn - 1;
				}

				var range = new Range(this, fromR, toR, fromC, toC);
				range.Normalize();

				return range;
			}
			set
			{
				this.set_Value(null, value);
			}
		}

		/// <summary>
		/// Returns a String value that represents the range reference in the language of the macro.
		/// </summary>
		/// <param name="RowAbsolute">Boolean, default true</param>
		/// <param name="ColumnAbsolute">Boolean, default true</param>
		/// <param name="ReferenceStyle"></param>
		/// <param name="External">ignored</param>
		/// <param name="RelativeTo">Range</param>
		/// <returns></returns>
		[DispId(236)]
		public string get_Address([Optional]object RowAbsolute, [Optional]object ColumnAbsolute, [Optional]Microsoft.Office.Interop.Excel.XlReferenceStyle ReferenceStyle, [Optional]object External, [Optional]object RelativeTo)
		{
			if (RowAbsolute == Type.Missing)
			{
				RowAbsolute = true;
			}
			if (ColumnAbsolute == Type.Missing)
			{
				ColumnAbsolute = true;
			}
			if (ReferenceStyle.ToString().Equals("0"))
			{
				ReferenceStyle = Microsoft.Office.Interop.Excel.XlReferenceStyle.xlA1;
			}
			if (External == Type.Missing)
			{
				External = false;
			}

			if (ReferenceStyle == Microsoft.Office.Interop.Excel.XlReferenceStyle.xlR1C1)
			{
				var cellRelativeTo = new Cell(1, 1);

				if (RelativeTo is Range)
				{
					var rangeRelativeTo = RelativeTo as Range;
					cellRelativeTo = new Cell(rangeRelativeTo.fromRow, rangeRelativeTo.fromColumn);
				}

				if (IsSingleCell)
				{
					return new Cell(fromRow, fromColumn).GetAddressR1C1((bool) RowAbsolute, (bool) ColumnAbsolute, cellRelativeTo);
				}
				else if (IncludesAllRows && IncludesAllColumns)
				{
					return ((bool) RowAbsolute)
						? String.Format("R1:R{0}", Range.LastRow)
						: String.Format("R:R[{0}]", Range.LastRow - 1);
				}
				else if (IncludesAllRows)
				{
					if (fromColumn == toColumn)
					{
						return new Cell(0, fromColumn).GetAddressR1C1((bool) RowAbsolute, (bool) ColumnAbsolute, cellRelativeTo);
					}
					else
					{
						return String.Format("{0}:{1}",
							new Cell(0, fromColumn).GetAddressR1C1((bool) RowAbsolute, (bool) ColumnAbsolute, cellRelativeTo),
							new Cell(0, toColumn).GetAddressR1C1((bool) RowAbsolute, (bool) ColumnAbsolute, cellRelativeTo));
					}
				}
				else if (IncludesAllColumns)
				{
					if (fromRow == toRow)
					{
						return new Cell(fromRow, 0).GetAddressR1C1((bool) RowAbsolute, (bool) ColumnAbsolute, cellRelativeTo);
					}
					else
					{
						return String.Format("{0}:{1}",
							new Cell(fromRow, 0).GetAddressR1C1((bool) RowAbsolute, (bool) ColumnAbsolute, cellRelativeTo),
							new Cell(toRow, 0).GetAddressR1C1((bool) RowAbsolute, (bool) ColumnAbsolute, cellRelativeTo));
					}
				}
				else
				{
					return String.Format("{0}:{1}",
						new Cell(fromRow, fromColumn).GetAddressR1C1((bool) RowAbsolute, (bool) ColumnAbsolute, cellRelativeTo),
						new Cell(toRow, toColumn).GetAddressR1C1((bool) RowAbsolute, (bool) ColumnAbsolute, cellRelativeTo));
				}
			}
			else
			{
				if (IsSingleCell)
				{
					return new Cell(fromRow, fromColumn).GetAddressA1((bool) RowAbsolute, (bool) ColumnAbsolute);
				}
				else if (IncludesAllRows && IncludesAllColumns)
				{
					return ((bool) RowAbsolute)
						? String.Format("$1:${0}", Range.LastRow)
						: String.Format("1:{0}", Range.LastRow);
				}
				else
				{
					return String.Format("{0}:{1}",
						new Cell(IncludesAllRows ? 0 : fromRow, IncludesAllColumns ? 0 : this.fromColumn).GetAddressA1((bool) RowAbsolute, (bool) ColumnAbsolute),
						new Cell(IncludesAllRows ? 0 : toRow, IncludesAllColumns ? 0 : this.toColumn).GetAddressA1((bool) RowAbsolute, (bool) ColumnAbsolute));
				}
			}
		}

		/// <summary>
		/// Returns a Borders collection that represents the borders of a style or a range of cells (including a range defined as part of a conditional format).
		/// </summary>
		[DispId(435)]
		public Microsoft.Office.Interop.Excel.Borders Borders
		{
			get { return style.Borders; }
		}

		/// <summary>
		/// Returns a Range object that represents the cells in the specified range.
		/// </summary>
		[DispId(238)]
		public Microsoft.Office.Interop.Excel.Range Cells
		{
			get
			{
				return (type == RangeType.Cells) ? this : new Range(this, RangeType.Cells);
			}
		}

		/// <summary>
		/// Returns the number of the first column in the first area in the specified range. Read-only.
		/// </summary>
		[DispId(240)]
		public int Column
		{
			get
			{
				return fromColumn;
			}
		}

		/// <summary>
		/// Returns a Range object that represents the columns in the specified range.
		/// </summary>
		[DispId(241)]
		public Microsoft.Office.Interop.Excel.Range Columns
		{
			get
			{
				return (type == RangeType.Columns) ? this : new Range(this, RangeType.Columns);
			}
		}

		/// <summary>
		/// Returns or sets the width of all columns in the specified range. Read/write.
		/// </summary>
		[DispId(242)]
		public dynamic ColumnWidth
		{
			get
			{
				return ScriptService.call("getColumnWidth", Document.Id, Sheet.Name, fromColumn, toColumn);
			}
			set
			{
				if (value is int || value is long)
				{
					ScriptService.call("setColumnWidth", Document.Id, Sheet.Name, fromColumn, toColumn, value);
				}
			}
		}

		/// <summary>
		/// Returns an int value that represents the number of objects in the collection.
		/// </summary>
		[DispId(118)]
		public int Count
		{
			get
			{
				switch (type)
				{
				case RangeType.Cells:
					return (toRow - fromRow + 1) * (toColumn - fromColumn + 1);
				case RangeType.Columns:
					return toColumn - fromColumn + 1;
				case RangeType.Rows:
					return toRow - fromRow + 1;
				default:
					return 0;
				}
			}
		}

		/// <summary>
		/// Returns a Range object that represents the entire column (or columns) that contains the specified range. Read-only.
		/// </summary>
		[DispId(246)]
		public Microsoft.Office.Interop.Excel.Range EntireColumn
		{
			get
			{
				var range = new Range(this, RangeType.Columns);
				range.fromRow = 1;
				range.toRow = Range.LastRow;
				return range;
			}
		}

		/// <summary>
		/// Returns a Range object that represents the entire row (or rows) that contains the specified range. Read-only.
		/// </summary>
		[DispId(247)]
		public Microsoft.Office.Interop.Excel.Range EntireRow
		{
			get
			{
				var range = new Range(this, RangeType.Rows);
				range.fromColumn = 1;
				range.toColumn = Range.LastColumn;
				return range;
			}
		}

		/// <summary>
		/// Returns a Font object that represents the font of the specified object.
		/// </summary>
		[DispId(146)]
		public Microsoft.Office.Interop.Excel.Font Font
		{
			get { return style.Font; }
		}

		/// <summary>
		/// Returns or sets a String value that represents the object's formula in A1-style notation and in the macro language.
		/// </summary>
		[DispId(261)]
		public dynamic Formula
		{
			get
			{
				return ScriptService.call("getFormula", this);
			}
			set
			{
				if (value is string)
				{
					ScriptService.call("setFormula", this, value);
				}
			}
		}

		/// <summary>
		/// Returns or sets the formula for the object, using R1C1-style notation in the language of the macro. Read/write.
		/// </summary>
		[DispId(264)]
		public dynamic FormulaR1C1
		{
			get
			{
				return ScriptService.call("getFormulaR1C1", this);
			}
			set
			{
				if (value is string)
				{
					ScriptService.call("setFormulaR1C1", this, value);
				}
			}
		}

		/// <summary>
		/// Returns or sets a XlHAlign value that represents the horizontal alignment for the specified object.
		/// </summary>
		[DispId(136)]
		public dynamic HorizontalAlignment
		{
			get
			{
				return style.HorizontalAlignment;
			}
			set
			{
				style.HorizontalAlignment = value;
			}
		}

		protected bool IncludesAllColumns
		{
			get
			{
				return (fromColumn == 1 && toColumn == Range.LastColumn);
			}
		}

		protected bool IncludesAllRows
		{
			get
			{
				return (fromRow == 1 && toRow == Range.LastRow);
			}
		}

		/// <summary>
		/// Returns an Interior object that represents the interior of the specified object.
		/// </summary>
		[DispId(129)]
		public Microsoft.Office.Interop.Excel.Interior Interior
		{
			get { return style.Interior; }
		}

		protected bool IsSingleCell
		{
			get
			{
				return (fromRow == toRow && fromColumn == toColumn);
			}
		}

		protected bool IsValid
		{
			get
			{
				return (fromRow >= 1 && fromColumn >= 1 && toRow <= Range.LastRow && toColumn <= Range.LastColumn);
			}
		}

		/// <summary>
		/// Returns a Range object that represents a range at an offset to the specified range.
		/// </summary>
		/// <param name="RowIndex"></param>
		/// <param name="ColumnIndex"></param>
		/// <returns></returns>
		[DispId(170)]
		dynamic Microsoft.Office.Interop.Excel.Range.get_Item(object RowIndex, [Optional]object ColumnIndex)
		{
			return this[RowIndex, ColumnIndex];
		}

		/// <summary>
		/// Sets a value of the specified range.
		/// </summary>
		/// <param name="RowIndex"></param>
		/// <param name="ColumnIndex"></param>
		/// <param name="value"></param>
		[DispId(170)]
		void Microsoft.Office.Interop.Excel.Range.set_Item(object RowIndex, [Optional]object ColumnIndex, [Optional]object value)
		{
			this[RowIndex, ColumnIndex] = value;
		}

		/// <summary>
		/// Returns or sets a String value that represents the format code for the object.
		/// </summary>
		[DispId(193)]
		public dynamic NumberFormat
		{
			get
			{
				return style.NumberFormat;
			}
			set
			{
				style.NumberFormat = value;
			}
		}

		/// <summary>
		/// Returns a Range object that represets a range that's offset from the specified range.
		/// </summary>
		/// <param name="RowOffset"></param>
		/// <param name="ColumnOffset"></param>
		/// <returns></returns>
		[DispId(254)]
		public Microsoft.Office.Interop.Excel.Range get_Offset([Optional]object RowOffset, [Optional]object ColumnOffset)
		{
			if (RowOffset == Type.Missing)
			{
				RowOffset = 0;
			}
			if (ColumnOffset == Type.Missing)
			{
				ColumnOffset = 0;
			}

			int rowOffset;
			int columnOffset;
			bool rowIsNum = Int32.TryParse(RowOffset.ToString(), out rowOffset);
			bool columnIsNum = Int32.TryParse(ColumnOffset.ToString(), out columnOffset);

			if (rowIsNum && columnIsNum)
			{
				var range = new Range(this, type);
				range.fromRow += rowOffset;
				range.toRow += rowOffset;
				range.fromColumn += columnOffset;
				range.toColumn += columnOffset;

				if (range.IsValid)
				{
					return range;
				}
			}

			throw new InvalidOperationException();
		}

		/// <summary>
		/// Returns a Range object that represents a cell or a range of cells.
		/// </summary>
		/// <param name="Cell1"></param>
		/// <param name="Cell2"></param>
		/// <returns></returns>
		[DispId(197)]
		public Microsoft.Office.Interop.Excel.Range get_Range(object Cell1, [Optional]object Cell2)
		{
			int fromR = 0;
			int toR = 0;
			int fromC = 0;
			int toC = 0;

			if (Cell1 is String && Cell2 == Type.Missing)
			{
				string[] addresses = (Cell1 as String).Split(':');

				var fromCell = new Cell(addresses[0]);
				fromR = fromCell.Row;
				fromC = fromCell.Column;

				if (addresses.Length == 2)
				{
					var toCell = new Cell(addresses[1]);
					toR = toCell.Row;
					toC = toCell.Column;
				}
				else
				{
					toR = fromR;
					toC = fromC;
				}

				if (fromR == 0 && toR == 0 && fromRow == 1)
				{
					fromR = 1;
					toR = Range.LastRow;
				}
				if (fromC == 0 && toC == 0 && fromColumn == 1)
				{
					fromC = 1;
					toC = Range.LastColumn;
				}
			}
			else
			{
				if (Cell1 is String)
				{
					var fromCell = new Cell(Cell1 as String);
					fromR = fromCell.Row;
					fromC = fromCell.Column;
				}
				else if (Cell1 is Range)
				{
					var fromCell = Cell1 as Range;
					fromR = fromCell.fromRow;
					fromC = fromCell.fromColumn;
				}

				if (Cell2 is String)
				{
					var toCell = new Cell(Cell2 as String);
					toR = toCell.Row;
					toC = toCell.Column;
				}
				else if (Cell2 is Range)
				{
					var toCell = Cell2 as Range;
					toR = toCell.toRow;
					toC = toCell.toColumn;
				}
			}

			if (fromR == 0 || toR == 0 || fromC == 0 || toC == 0)
			{
				throw new InvalidOperationException();
			}

			if (toR != Range.LastRow)
			{
				fromR += fromRow - 1;
				toR += fromRow - 1;
			}
			if (toC != Range.LastColumn)
			{
				fromC += fromColumn - 1;
				toC += fromColumn - 1;
			}

			var range = new Range(this, fromR, toR, fromC, toC);
			range.Normalize();

			return range;
		}

		/// <summary>
		/// Returns the number of the first row of the first area in the range. Read-only.
		/// </summary>
		[DispId(257)]
		public int Row
		{
			get
			{
				return fromRow;
			}
		}

		/// <summary>
		/// Returns or sets the height of the first row in the range specified, measured in pixels. Read/write.
		/// </summary>
		[DispId(272)]
		public dynamic RowHeight
		{
			get
			{
				return ScriptService.call("getRowHeight", Document.Id, Sheet.Name, fromRow, toRow);
			}
			set
			{
				if (value is int || value is long)
				{
					ScriptService.call("setRowHeight", Document.Id, Sheet.Name, fromRow, toRow, value);
				}
			}
		}

		/// <summary>
		/// Returns a Range object that represents the rows in the specified range. Read-only Range object.
		/// </summary>
		[DispId(258)]
		public Microsoft.Office.Interop.Excel.Range Rows
		{
			get
			{
				return (type == RangeType.Rows) ? this : new Range(this, RangeType.Rows);
			}
		}

		/// <summary>
		/// Returns or sets a Style object, that represents the style of the specified range.
		/// </summary>
		[DispId(260)]
		public dynamic Style
		{
			get
			{
				return style;
			}
			set { }
		}

		/// <summary>
		/// Returns or sets the text for the specified object. Read-only String.
		/// </summary>
		[DispId(138)]
		public dynamic Text
		{
			get
			{
				return ScriptService.call("getDisplayValue", this);
			}
		}

		/// <summary>
		/// Returns a value of the specified range.
		/// </summary>
		/// <param name="RangeValueDataType"></param>
		/// <returns></returns>
		[DispId(6)]
		public dynamic get_Value([Optional]object RangeValueDataType)
		{
			return ScriptService.call("getValue", this);
		}

		/// <summary>
		/// Sets a value of the specified range.
		/// </summary>
		/// <param name="RangeValueDataType"></param>
		/// <param name="value"></param>
		[DispId(6)]
		public void set_Value([Optional]object RangeValueDataType, [Optional]object value)
		{
			(Worksheet as Worksheet).ActiveCell = new Range(this, fromRow, fromRow, fromColumn, fromColumn);
			ScriptService.call("setValue", this, value.ToString());
		}

		/// <summary>
		/// Returns or sets the cell value. Read/write.
		/// </summary>
		[DispId(1388)]
		public dynamic Value2
		{
			get
			{
				Service.Load();
				Cell tlCell = Service[fromRow, fromColumn];
				string numericValue = tlCell.Entry.NumericValue;
				string inputValue = tlCell.Entry.InputValue;
				for (int r = fromRow; r <= toRow; r++)
				{
					for (int c = fromColumn; c <= toColumn; c++)
					{
						if (Service[r, c].Entry.InputValue != inputValue)
						{
							return null;
						}
					}
				}
				return numericValue == null ? inputValue : numericValue;
			}
			set
			{
				this.set_Value(null, value);
			}
		}

		/// <summary>
		/// Returns or sets a XlVAlign value that represents the vertical alignment of the specified object.
		/// </summary>
		[DispId(137)]
		public dynamic VerticalAlignment
		{
			get
			{
				return style.VerticalAlignment;
			}
			set
			{
				style.VerticalAlignment = value;
			}
		}

		/// <summary>
		/// Returns a Worksheet object that represents the worksheet containing the specified range. Read-only.
		/// </summary>
		[DispId(348)]
		public Microsoft.Office.Interop.Excel.Worksheet Worksheet { get; private set; }

		/// <summary>
		/// Returns or sets a Boolean value that indicates if text is wrapped in cells.
		/// </summary>
		[DispId(276)]
		public dynamic WrapText
		{
			get
			{
				return style.WrapText;
			}
			set
			{
				style.WrapText = value;
			}
		}

		#endregion

		#region methods

		/// <summary>
		/// Adds a border to a range and sets the Color and LineStyle properties for the new border.
		/// </summary>
		/// <param name="LineStyle">XlLineStyle</param>
		/// <param name="Weight">ignored</param>
		/// <param name="ColorIndex">ignored</param>
		/// <param name="Color"></param>
		/// <returns>Borders object</returns>
		[DispId(1067)]
		public dynamic BorderAround(object LineStyle = null, Microsoft.Office.Interop.Excel.XlBorderWeight Weight = Microsoft.Office.Interop.Excel.XlBorderWeight.xlThin, Microsoft.Office.Interop.Excel.XlColorIndex ColorIndex = Microsoft.Office.Interop.Excel.XlColorIndex.xlColorIndexAutomatic, object Color = null)
		{
			if (LineStyle != null && LineStyle is Microsoft.Office.Interop.Excel.XlLineStyle)
			{
				var lineStyle = (Microsoft.Office.Interop.Excel.XlLineStyle) LineStyle;
				if (lineStyle != Microsoft.Office.Interop.Excel.XlLineStyle.xlLineStyleNone)
				{
					var color = new ExcelColor(Color);
					new Border[] {
						Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeTop] as Border,
						Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft] as Border,
						Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeBottom] as Border,
						Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight] as Border
					}.ToList().ForEach(border => {
						border.SetColor(color);
						border.SetLineStyle(lineStyle);
					});
					(Borders as Borders).UpdateBorders();
				}
			}
			return Borders;
		}

		/// <summary>
		/// Clears the entire object.
		/// </summary>
		/// <returns>Boolean</returns>
		[DispId(111)]
		public dynamic Clear()
		{
			ScriptService.call("clearRange", this);

			return true;
		}

		/// <summary>
		/// Clears the formulas from the range.
		/// </summary>
		/// <returns>Boolean</returns>
		[DispId(113)]
		public dynamic ClearContents()
		{
			ScriptService.call("clearRangeContent", this);

			return true;
		}

		/// <summary>
		/// Clears the formatting of the object.
		/// </summary>
		/// <returns>Boolean</returns>
		[DispId(112)]
		public dynamic ClearFormats()
		{
			ScriptService.call("clearRangeFormat", this);

			return true;
		}

		/// <summary>
		/// Inserts a cell or a range of cells into the worksheet and shifts other cells away to make space.
		/// </summary>
		/// <param name="Shift">XlInsertShiftDirection</param>
		/// <param name="CopyOrigin">ignored</param>
		/// <returns>Boolean</returns>
		[DispId(252)]
		public dynamic Insert([Optional]object Shift, [Optional]object CopyOrigin)
		{
			var shiftDirection = (Shift is Microsoft.Office.Interop.Excel.XlInsertShiftDirection)
				? (Microsoft.Office.Interop.Excel.XlInsertShiftDirection) Shift
				: Microsoft.Office.Interop.Excel.XlInsertShiftDirection.xlShiftToRight;
			if (shiftDirection == Microsoft.Office.Interop.Excel.XlInsertShiftDirection.xlShiftToRight)
			{
				ScriptService.call("shiftRangeToRight", this);
			}
			else
			{
				ScriptService.call("shiftRangeDown", this);
			}

			return true;
		}

		/// <summary>
		/// Creates a merged cell from the specified Range object.
		/// </summary>
		/// <param name="Across">Boolean</param>
		[DispId(564)]
		public void Merge([Optional]object Across)
		{
			bool across = (Across is bool)
				? (bool) Across
				: false;

			if (across)
			{
				ScriptService.call("mergeRangeAcross", this);
			}
			else
			{
				ScriptService.call("mergeRange", this);
			}
		}

		protected void Normalize()
		{
			fromRow = (fromRow < 1) ? 1 : fromRow;
			fromColumn = (fromColumn < 1) ? 1 : fromColumn;
			toRow = (toRow > Range.LastRow) ? Range.LastRow : toRow;
			toColumn = (toColumn > Range.LastColumn) ? Range.LastColumn : toColumn;
		}

		/// <summary>
		/// Separates a merged area into individual cells.
		/// </summary>
		[DispId(1384)]
		public void UnMerge()
		{
			ScriptService.call("unmergeRange", this);
		}

		#endregion

		#region unused members

		[DispId(304)]
		public dynamic Activate()
		{
			throw new NotImplementedException();
		}

		[DispId(1389)]
		public Microsoft.Office.Interop.Excel.Comment AddComment([Optional]object Text)
		{
			throw new NotImplementedException();
		}

		[DispId(1063)]
		public dynamic AddIndent
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(876)]
		public dynamic AdvancedFilter(Microsoft.Office.Interop.Excel.XlFilterAction Action, [Optional]object CriteriaRange, [Optional]object CopyToRange, [Optional]object Unique)
		{
			throw new NotImplementedException();
		}

		[DispId(2020)]
		public bool AllowEdit
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(441)]
		public dynamic ApplyNames([Optional]object Names, [Optional]object IgnoreRelativeAbsolute, [Optional]object UseRowColumnNames, [Optional]object OmitColumn, [Optional]object OmitRow, Microsoft.Office.Interop.Excel.XlApplyNamesOrder Order, [Optional]object AppendLast)
		{
			throw new NotImplementedException();
		}

		[DispId(448)]
		public dynamic ApplyOutlineStyles()
		{
			throw new NotImplementedException();
		}

		[DispId(568)]
		public Microsoft.Office.Interop.Excel.Areas Areas
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1185)]
		public string AutoComplete(string String)
		{
			throw new NotImplementedException();
		}

		[DispId(449)]
		public dynamic AutoFill(Microsoft.Office.Interop.Excel.Range Destination, Microsoft.Office.Interop.Excel.XlAutoFillType Type = Microsoft.Office.Interop.Excel.XlAutoFillType.xlFillDefault)
		{
			throw new NotImplementedException();
		}

		[DispId(793)]
		public dynamic AutoFilter([Optional]object Field, [Optional]object Criteria1, Microsoft.Office.Interop.Excel.XlAutoFilterOperator Operator, [Optional]object Criteria2, [Optional]object VisibleDropDown)
		{
			throw new NotImplementedException();
		}

		[DispId(237)]
		public dynamic AutoFit()
		{
			throw new NotImplementedException();
		}

		[DispId(114)]
		public dynamic AutoFormat(Microsoft.Office.Interop.Excel.XlRangeAutoFormat Format, [Optional]object Number, [Optional]object Font, [Optional]object Alignment, [Optional]object Border, [Optional]object Pattern, [Optional]object Width)
		{
			throw new NotImplementedException();
		}

		[DispId(1036)]
		public dynamic AutoOutline()
		{
			throw new NotImplementedException();
		}

		[DispId(279)]
		public dynamic Calculate()
		{
			throw new NotImplementedException();
		}

		[DispId(2364)]
		public dynamic CalculateRowMajorOrder()
		{
			throw new NotImplementedException();
		}

		[DispId(505)]
		public dynamic CheckSpelling([Optional]object CustomDictionary, [Optional]object IgnoreUppercase, [Optional]object AlwaysSuggest, [Optional]object SpellLang)
		{
			throw new NotImplementedException();
		}

		[DispId(1390)]
		public void ClearComments()
		{
			throw new NotImplementedException();
		}

		[DispId(239)]
		public dynamic ClearNotes()
		{
			throw new NotImplementedException();
		}

		[DispId(1037)]
		public dynamic ClearOutline()
		{
			throw new NotImplementedException();
		}

		[DispId(510)]
		public Microsoft.Office.Interop.Excel.Range ColumnDifferences(object Comparison)
		{
			throw new NotImplementedException();
		}

		[DispId(910)]
		public Microsoft.Office.Interop.Excel.Comment Comment
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(482)]
		public dynamic Consolidate([Optional]object Sources, [Optional]object Function, [Optional]object TopRow, [Optional]object LeftColumn, [Optional]object CreateLinks)
		{
			throw new NotImplementedException();
		}

		[DispId(551)]
		public dynamic Copy([Optional]object Destination)
		{
			throw new NotImplementedException();
		}

		[DispId(1152)]
		public int CopyFromRecordset(object Data, [Optional]object MaxRows, [Optional]object MaxColumns)
		{
			throw new NotImplementedException();
		}

		[DispId(213)]
		public dynamic CopyPicture(Microsoft.Office.Interop.Excel.XlPictureAppearance Appearance, Microsoft.Office.Interop.Excel.XlCopyPictureFormat Format = Microsoft.Office.Interop.Excel.XlCopyPictureFormat.xlPicture)
		{
			throw new NotImplementedException();
		}

		[DispId(2499)]
		public dynamic CountLarge
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(457)]
		public dynamic CreateNames([Optional]object Top, [Optional]object Left, [Optional]object Bottom, [Optional]object Right)
		{
			throw new NotImplementedException();
		}

		[DispId(458)]
		public dynamic CreatePublisher([Optional]object Edition, Microsoft.Office.Interop.Excel.XlPictureAppearance Appearance, [Optional]object ContainsPICT, [Optional]object ContainsBIFF, [Optional]object ContainsRTF, [Optional]object ContainsVALU)
		{
			throw new NotImplementedException();
		}

		[DispId(149)]
		public Microsoft.Office.Interop.Excel.XlCreator Creator
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(501)]
		public Microsoft.Office.Interop.Excel.Range CurrentArray
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(243)]
		public Microsoft.Office.Interop.Excel.Range CurrentRegion
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(565)]
		public dynamic Cut([Optional]object Destination)
		{
			throw new NotImplementedException();
		}

		[DispId(464)]
		public dynamic DataSeries([Optional]object Rowcol, Microsoft.Office.Interop.Excel.XlDataSeriesType Type, Microsoft.Office.Interop.Excel.XlDataSeriesDate Date, [Optional]object Step, [Optional]object Stop, [Optional]object Trend)
		{
			throw new NotImplementedException();
		}

		[DispId(117)]
		public dynamic Delete([Optional]object Shift)
		{
			throw new NotImplementedException();
		}

		[DispId(543)]
		public Microsoft.Office.Interop.Excel.Range Dependents
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(245)]
		public dynamic DialogBox()
		{
			throw new NotImplementedException();
		}

		[DispId(545)]
		public Microsoft.Office.Interop.Excel.Range DirectDependents
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(546)]
		public Microsoft.Office.Interop.Excel.Range DirectPrecedents
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(2014)]
		public void Dirty()
		{
			throw new NotImplementedException();
		}

		[DispId(1131)]
		public dynamic EditionOptions(Microsoft.Office.Interop.Excel.XlEditionType Type, Microsoft.Office.Interop.Excel.XlEditionOptionsOption Option, [Optional]object Name, [Optional]object Reference, Microsoft.Office.Interop.Excel.XlPictureAppearance Appearance, Microsoft.Office.Interop.Excel.XlPictureAppearance ChartSize, [Optional]object Format)
		{
			throw new NotImplementedException();
		}

		[DispId(2015)]
		public Microsoft.Office.Interop.Excel.Errors Errors
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(2493)]
		public void ExportAsFixedFormat(Microsoft.Office.Interop.Excel.XlFixedFormatType Type, [Optional]object Filename, [Optional]object Quality, [Optional]object IncludeDocProperties, [Optional]object IgnorePrintAreas, [Optional]object From, [Optional]object To, [Optional]object OpenAfterPublish, [Optional]object FixedFormatExtClassPtr)
		{
			throw new NotImplementedException();
		}

		[DispId(248)]
		public dynamic FillDown()
		{
			throw new NotImplementedException();
		}

		[DispId(249)]
		public dynamic FillLeft()
		{
			throw new NotImplementedException();
		}

		[DispId(250)]
		public dynamic FillRight()
		{
			throw new NotImplementedException();
		}

		[DispId(251)]
		public dynamic FillUp()
		{
			throw new NotImplementedException();
		}

		[DispId(398)]
		public Microsoft.Office.Interop.Excel.Range Find(object What, [Optional]object After, [Optional]object LookIn, [Optional]object LookAt, [Optional]object SearchOrder, Microsoft.Office.Interop.Excel.XlSearchDirection SearchDirection, [Optional]object MatchCase, [Optional]object MatchByte, [Optional]object SearchFormat)
		{
			throw new NotImplementedException();
		}

		[DispId(399)]
		public Microsoft.Office.Interop.Excel.Range FindNext([Optional]object After)
		{
			throw new NotImplementedException();
		}

		[DispId(400)]
		public Microsoft.Office.Interop.Excel.Range FindPrevious([Optional]object After)
		{
			throw new NotImplementedException();
		}

		[DispId(1392)]
		public Microsoft.Office.Interop.Excel.FormatConditions FormatConditions
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(586)]
		public dynamic FormulaArray
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(262)]
		public dynamic FormulaHidden
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1380)]
		public Microsoft.Office.Interop.Excel.XlFormulaLabel FormulaLabel
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(263)]
		public dynamic FormulaLocal
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(265)]
		public dynamic FormulaR1C1Local
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(571)]
		public dynamic FunctionWizard()
		{
			throw new NotImplementedException();
		}

		[DispId(-4)]
		public System.Collections.IEnumerator GetEnumerator()
		{
			throw new NotImplementedException();
		}

		[DispId(472)]
		public bool GoalSeek(object Goal, Microsoft.Office.Interop.Excel.Range ChangingCell)
		{
			throw new NotImplementedException();
		}

		[DispId(46)]
		public dynamic Group([Optional]object Start, [Optional]object End, [Optional]object By, [Optional]object Periods)
		{
			throw new NotImplementedException();
		}

		[DispId(266)]
		public dynamic HasArray
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(267)]
		public dynamic HasFormula
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(123)]
		public dynamic Height
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(268)]
		public dynamic Hidden
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1393)]
		public Microsoft.Office.Interop.Excel.Hyperlinks Hyperlinks
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1813)]
		public string ID
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(201)]
		public dynamic IndentLevel
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1381)]
		public void InsertIndent(int InsertAmount)
		{
			throw new NotImplementedException();
		}

		[DispId(495)]
		public dynamic Justify()
		{
			throw new NotImplementedException();
		}

		[DispId(127)]
		public dynamic Left
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1187)]
		public int ListHeaderRows
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(253)]
		public dynamic ListNames()
		{
			throw new NotImplementedException();
		}

		[DispId(2257)]
		public Microsoft.Office.Interop.Excel.ListObject ListObject
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(691)]
		public Microsoft.Office.Interop.Excel.XlLocationInTable LocationInTable
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(269)]
		public dynamic Locked
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1385)]
		public Microsoft.Office.Interop.Excel.Range MergeArea
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(208)]
		public dynamic MergeCells
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2123)]
		public string MDX
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(110)]
		public dynamic Name
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1032)]
		public dynamic NavigateArrow([Optional]object TowardPrecedent, [Optional]object ArrowNumber, [Optional]object LinkNumber)
		{
			throw new NotImplementedException();
		}

		[DispId(502)]
		public Microsoft.Office.Interop.Excel.Range Next
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1127)]
		public string NoteText([Optional]object Text, [Optional]object Start, [Optional]object Length)
		{
			throw new NotImplementedException();
		}

		[DispId(1097)]
		public dynamic NumberFormatLocal
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(134)]
		public dynamic Orientation
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(271)]
		public dynamic OutlineLevel
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(255)]
		public int PageBreak
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(150)]
		public dynamic Parent
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(477)]
		public dynamic Parse([Optional]object ParseLine, [Optional]object Destination)
		{
			throw new NotImplementedException();
		}

		[DispId(1928)]
		public dynamic PasteSpecial(Microsoft.Office.Interop.Excel.XlPasteType Paste, Microsoft.Office.Interop.Excel.XlPasteSpecialOperation Operation, [Optional]object SkipBlanks, [Optional]object Transpose)
		{
			throw new NotImplementedException();
		}

		[DispId(1391)]
		public Microsoft.Office.Interop.Excel.Phonetic Phonetic
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1811)]
		public Microsoft.Office.Interop.Excel.Phonetics Phonetics
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(2013)]
		public Microsoft.Office.Interop.Excel.PivotCell PivotCell
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(731)]
		public Microsoft.Office.Interop.Excel.PivotField PivotField
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(740)]
		public Microsoft.Office.Interop.Excel.PivotItem PivotItem
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(716)]
		public Microsoft.Office.Interop.Excel.PivotTable PivotTable
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(544)]
		public Microsoft.Office.Interop.Excel.Range Precedents
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(504)]
		public dynamic PrefixCharacter
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(503)]
		public Microsoft.Office.Interop.Excel.Range Previous
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1772)]
		public dynamic PrintOut([Optional]object From, [Optional]object To, [Optional]object Copies, [Optional]object Preview, [Optional]object ActivePrinter, [Optional]object PrintToFile, [Optional]object Collate, [Optional]object PrToFileName)
		{
			throw new NotImplementedException();
		}

		[DispId(2361)]
		public dynamic PrintOutEx([Optional]object From, [Optional]object To, [Optional]object Copies, [Optional]object Preview, [Optional]object ActivePrinter, [Optional]object PrintToFile, [Optional]object Collate, [Optional]object PrToFileName)
		{
			throw new NotImplementedException();
		}

		[DispId(281)]
		public dynamic PrintPreview([Optional]object EnableChanges)
		{
			throw new NotImplementedException();
		}

		[DispId(1386)]
		public Microsoft.Office.Interop.Excel.QueryTable QueryTable
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(975)]
		public int ReadingOrder
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2492)]
		public void RemoveDuplicates([Optional]object Columns, Microsoft.Office.Interop.Excel.XlYesNoGuess Header)
		{
			throw new NotImplementedException();
		}

		[DispId(883)]
		public dynamic RemoveSubtotal()
		{
			throw new NotImplementedException();
		}

		[DispId(226)]
		public bool Replace(object What, object Replacement, [Optional]object LookAt, [Optional]object SearchOrder, [Optional]object MatchCase, [Optional]object MatchByte, [Optional]object SearchFormat, [Optional]object ReplaceFormat)
		{
			throw new NotImplementedException();
		}

		[DispId(511)]
		public Microsoft.Office.Interop.Excel.Range RowDifferences(object Comparison)
		{
			throw new NotImplementedException();
		}

		[DispId(259)]
		public dynamic Run([Optional]object Arg1, [Optional]object Arg2, [Optional]object Arg3, [Optional]object Arg4, [Optional]object Arg5, [Optional]object Arg6, [Optional]object Arg7, [Optional]object Arg8, [Optional]object Arg9, [Optional]object Arg10, [Optional]object Arg11, [Optional]object Arg12, [Optional]object Arg13, [Optional]object Arg14, [Optional]object Arg15, [Optional]object Arg16, [Optional]object Arg17, [Optional]object Arg18, [Optional]object Arg19, [Optional]object Arg20, [Optional]object Arg21, [Optional]object Arg22, [Optional]object Arg23, [Optional]object Arg24, [Optional]object Arg25, [Optional]object Arg26, [Optional]object Arg27, [Optional]object Arg28, [Optional]object Arg29, [Optional]object Arg30)
		{
			throw new NotImplementedException();
		}

		[DispId(235)]
		public dynamic Select()
		{
			throw new NotImplementedException();
		}

		[DispId(2491)]
		public Microsoft.Office.Interop.Excel.Actions ServerActions
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1812)]
		public void SetPhonetic()
		{
			throw new NotImplementedException();
		}

		[DispId(496)]
		public dynamic Show()
		{
			throw new NotImplementedException();
		}

		[DispId(877)]
		public dynamic ShowDependents([Optional]object Remove)
		{
			throw new NotImplementedException();
		}

		[DispId(585)]
		public dynamic ShowDetail
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(878)]
		public dynamic ShowErrors()
		{
			throw new NotImplementedException();
		}

		[DispId(879)]
		public dynamic ShowPrecedents([Optional]object Remove)
		{
			throw new NotImplementedException();
		}

		[DispId(209)]
		public dynamic ShrinkToFit
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(2016)]
		public Microsoft.Office.Interop.Excel.SmartTags SmartTags
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(880)]
		public dynamic Sort([Optional]object Key1, Microsoft.Office.Interop.Excel.XlSortOrder Order1, [Optional]object Key2, [Optional]object Type, Microsoft.Office.Interop.Excel.XlSortOrder Order2, [Optional]object Key3, Microsoft.Office.Interop.Excel.XlSortOrder Order3, Microsoft.Office.Interop.Excel.XlYesNoGuess Header, [Optional]object OrderCustom, [Optional]object MatchCase, Microsoft.Office.Interop.Excel.XlSortOrientation Orientation, Microsoft.Office.Interop.Excel.XlSortMethod SortMethod, Microsoft.Office.Interop.Excel.XlSortDataOption DataOption1, Microsoft.Office.Interop.Excel.XlSortDataOption DataOption2, Microsoft.Office.Interop.Excel.XlSortDataOption DataOption3)
		{
			throw new NotImplementedException();
		}

		[DispId(881)]
		public dynamic SortSpecial(Microsoft.Office.Interop.Excel.XlSortMethod SortMethod, [Optional]object Key1, Microsoft.Office.Interop.Excel.XlSortOrder Order1, [Optional]object Type, [Optional]object Key2, Microsoft.Office.Interop.Excel.XlSortOrder Order2, [Optional]object Key3, Microsoft.Office.Interop.Excel.XlSortOrder Order3, Microsoft.Office.Interop.Excel.XlYesNoGuess Header, [Optional]object OrderCustom, [Optional]object MatchCase, Microsoft.Office.Interop.Excel.XlSortOrientation Orientation, Microsoft.Office.Interop.Excel.XlSortDataOption DataOption1, Microsoft.Office.Interop.Excel.XlSortDataOption DataOption2, Microsoft.Office.Interop.Excel.XlSortDataOption DataOption3)
		{
			throw new NotImplementedException();
		}

		[DispId(916)]
		public Microsoft.Office.Interop.Excel.SoundNote SoundNote
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(2017)]
		public void Speak([Optional]object SpeakDirection, [Optional]object SpeakFormulas)
		{
			throw new NotImplementedException();
		}

		[DispId(410)]
		public Microsoft.Office.Interop.Excel.Range SpecialCells(Microsoft.Office.Interop.Excel.XlCellType Type, [Optional]object Value)
		{
			throw new NotImplementedException();
		}

		[DispId(481)]
		public dynamic SubscribeTo(string Edition, Microsoft.Office.Interop.Excel.XlSubscribeToFormat Format = Microsoft.Office.Interop.Excel.XlSubscribeToFormat.xlSubscribeToText)
		{
			throw new NotImplementedException();
		}

		[DispId(882)]
		public dynamic Subtotal(int GroupBy, Microsoft.Office.Interop.Excel.XlConsolidationFunction Function, object TotalList, [Optional]object Replace, [Optional]object PageBreaks, Microsoft.Office.Interop.Excel.XlSummaryRow SummaryBelowData = Microsoft.Office.Interop.Excel.XlSummaryRow.xlSummaryBelow)
		{
			throw new NotImplementedException();
		}

		[DispId(273)]
		public dynamic Summary
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(497)]
		public dynamic Table([Optional]object RowInput, [Optional]object ColumnInput)
		{
			throw new NotImplementedException();
		}

		[DispId(1040)]
		public dynamic TextToColumns([Optional]object Destination, Microsoft.Office.Interop.Excel.XlTextParsingType DataType, Microsoft.Office.Interop.Excel.XlTextQualifier TextQualifier, [Optional]object ConsecutiveDelimiter, [Optional]object Tab, [Optional]object Semicolon, [Optional]object Comma, [Optional]object Space, [Optional]object Other, [Optional]object OtherChar, [Optional]object FieldInfo, [Optional]object DecimalSeparator, [Optional]object ThousandsSeparator, [Optional]object TrailingMinusNumbers)
		{
			throw new NotImplementedException();
		}

		[DispId(126)]
		public dynamic Top
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(244)]
		public dynamic Ungroup()
		{
			throw new NotImplementedException();
		}

		[DispId(274)]
		public dynamic UseStandardHeight
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(275)]
		public dynamic UseStandardWidth
		{
			get
			{
				throw new NotImplementedException();
			}
			set
			{
				throw new NotImplementedException();
			}
		}

		[DispId(1387)]
		public Microsoft.Office.Interop.Excel.Validation Validation
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(122)]
		public dynamic Width
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(2258)]
		public Microsoft.Office.Interop.Excel.XPath XPath
		{
			get { throw new NotImplementedException(); }
		}

		[DispId(1027)]
		public dynamic _PasteSpecial(Microsoft.Office.Interop.Excel.XlPasteType Paste, Microsoft.Office.Interop.Excel.XlPasteSpecialOperation Operation, [Optional]object SkipBlanks, [Optional]object Transpose)
		{
			throw new NotImplementedException();
		}

		[DispId(905)]
		public dynamic _PrintOut([Optional]object From, [Optional]object To, [Optional]object Copies, [Optional]object Preview, [Optional]object ActivePrinter, [Optional]object PrintToFile, [Optional]object Collate)
		{
			throw new NotImplementedException();
		}

		[DispId(437)]
		public string get_AddressLocal([Optional]object RowAbsolute, [Optional]object ColumnAbsolute, Microsoft.Office.Interop.Excel.XlReferenceStyle ReferenceStyle, [Optional]object External, [Optional]object RelativeTo)
		{
			throw new NotImplementedException();
		}

		[DispId(603)]
		public Microsoft.Office.Interop.Excel.Characters get_Characters([Optional]object Start, [Optional]object Length)
		{
			throw new NotImplementedException();
		}

		[DispId(500)]
		public Microsoft.Office.Interop.Excel.Range get_End(Microsoft.Office.Interop.Excel.XlDirection Direction)
		{
			throw new NotImplementedException();
		}

		[DispId(256)]
		public Microsoft.Office.Interop.Excel.Range get_Resize([Optional]object RowSize, [Optional]object ColumnSize)
		{
			throw new NotImplementedException();
		}

		#endregion

		#region excel version specific members
		/**/

		public void AllocateChanges()
		{
			throw new NotImplementedException();
		}

		public dynamic BorderAround2([OptionalAttribute]object LineStyle, [Optional]Microsoft.Office.Interop.Excel.XlBorderWeight Weight, [Optional]Microsoft.Office.Interop.Excel.XlColorIndex ColorIndex, [OptionalAttribute]object Color, [OptionalAttribute]object ThemeColor)
		{
			throw new NotImplementedException();
		}

		public void ClearHyperlinks()
		{
			throw new NotImplementedException();
		}

		public void DiscardChanges()
		{
			throw new NotImplementedException();
		}

		public Microsoft.Office.Interop.Excel.DisplayFormat DisplayFormat
		{
			get { throw new NotImplementedException(); }
		}

		public void FlashFill()
		{
			throw new NotImplementedException();
		}

		public Microsoft.Office.Interop.Excel.SparklineGroups SparklineGroups
		{
			get { throw new NotImplementedException(); }
		}

		/*//**/
		#endregion
	}
}
